import asyncio
import uuid
from typing import Optional, Union, Dict, Callable, List, Literal
from PySide6.QtWidgets import QFrame, QWidget, QHBoxLayout, QVBoxLayout, QLabel, QPushButton, QSizePolicy
from PySide6.QtGui import QIcon
from PySide6.QtCore import Signal, Property, Slot, QRunnable, QThreadPool, QTimer
from qtmui.hooks import State, useEffect
from ..py_iconify import PyIconify
from ..spacer import HSpacer
from ..typography import Typography
from ...qtmui_assets import QTMUI_ASSETS
from qtmui.material.styles import useTheme
from ..utils.validate_params import _validate_param

class WidgetSetter(QRunnable):
    def __init__(self):
        super().__init__()

    async def run(self, cls, widget):
        cls.layout().addWidget(widget)


class Collapse(QFrame):
    """
    A component that transitions its children in and out by collapsing or expanding its size.

    The `Collapse` component animates the visibility of its children by changing its height or width,
    supporting vertical or horizontal orientation. It supports all props of the Material-UI `Collapse`
    and `Transition` components, as well as additional props for customization.

    Parameters
    ----------
    addEndListener : State, Callable, or None, optional
        Custom transition end trigger. Called with the transitioning widget and a done callback.
        Default is None. Can be a `State` object for dynamic updates.
    child : State, QWidget, or None, optional
        A single child widget to be collapsed. Default is None.
        Can be a `State` object for dynamic updates.
    children : State, QWidget, List[QWidget], or None, optional
        The content node(s) to be collapsed. Takes precedence over `child`. Default is None.
        Can be a `State` object for dynamic updates.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    collapsedSize : State, int, or str, optional
        The size (height for vertical, width for horizontal) when collapsed. Default is "0px".
        Can be a `State` object for dynamic updates.
    component : State, str, or None, optional
        The component used for the root node (e.g., HTML element or custom component). Default is None.
        Can be a `State` object for dynamic updates.
    easing : State, str, dict, or None, optional
        The transition timing function (e.g., "ease", or {enter: "ease-in", exit: "ease-out"}).
        Default is None. Can be a `State` object for dynamic updates.
    id : State, str, or None, optional
        The identifier for the component. Default is None.
        Can be a `State` object for dynamic updates.
    isIn : State, bool, or None, optional
        If True, the component transitions in. Default is False.
        Can be a `State` object for dynamic updates.
    maximumHeight : State, int, or None, optional
        The maximum height of the component. Default is None.
        Can be a `State` object for dynamic updates.
    minimumHeight : State, int, or None, optional
        The minimum height of the component. Default is None.
        Can be a `State` object for dynamic updates.
    orientation : State or str, optional
        The transition orientation ("horizontal" or "vertical"). Default is "vertical".
        Can be a `State` object for dynamic updates.
    showToogleButton : State or bool, optional
        If True, displays a toggle button with title and icon. Default is True.
        Can be a `State` object for dynamic updates.
    startActionWidget : State, QWidget, or None, optional
        A widget to display at the start of the toggle button. Default is None.
        Can be a `State` object for dynamic updates.
    sx : State, dict, Callable, str, or None, optional
        System prop for CSS overrides and additional styles. Default is None.
        Can be a `State` object for dynamic updates.
    timeout : State, str, int, dict, or None, optional
        The duration for the transition (ms, "auto", or {appear, enter, exit}). Default is "auto".
        Can be a `State` object for dynamic updates.
    title : State or str, optional
        The title displayed in the toggle button. Default is "Page".
        Can be a `State` object for dynamic updates.
    unmountOnExit : State, bool, or None, optional
        If True, removes children from the layout when collapsed. Default is None.
        Can be a `State` object for dynamic updates.
    vExpanding : State, bool, or None, optional
        If True, sets expanding size policy (vertical or horizontal based on orientation).
        Default is None. Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to the parent `QFrame` class.

    Attributes
    ----------
    is_in_signal : Signal
        Signal emitted when the `isIn` state changes, carrying the new state.
    VALID_ORIENTATIONS : list[str]
        Valid values for the `orientation` parameter: ["horizontal", "vertical"].

    Notes
    -----
    - Props of the `Transition` component are supported (e.g., `addEndListener`).
    - The `children` prop takes precedence over `child`.
    - The `id`, `vExpanding`, `unmountOnExit`, `showToogleButton`, `title`, `minimumHeight`,
      `maximumHeight`, and `startActionWidget` props are specific to this implementation.

    Demos:
    - Collapse: https://qtmui.com/material-ui/qtmui-collapse/

    API Reference:
    - Collapse API: https://qtmui.com/material-ui/api/collapse/
    - Transition API: https://qtmui.com/material-ui/api/transition/
    """

    is_in_signal = Signal(bool)
    VALID_ORIENTATIONS = ["horizontal", "vertical"]

    def __init__(
        self,
        addEndListener: Optional[Union[State, Callable]] = None,
        child: Optional[Union[State, QWidget]] = None,
        children: Optional[Union[State, QWidget, List[QWidget]]] = None,
        classes: Optional[Union[State, Dict]] = None,
        collapsedSize: Union[State, int, str] = "0px",
        component: Optional[Union[State, str]] = None,
        easing: Optional[Union[State, str, Dict[str, str]]] = None,
        id: Optional[Union[State, str]] = None,
        isIn: Optional[Union[State, bool]] = False,
        maximumHeight: Optional[Union[State, int]] = None,
        minimumHeight: Optional[Union[State, int]] = None,
        orientation: Union[State, str] = "vertical",
        showToogleButton: Union[State, bool] = True,
        startActionWidget: Optional[Union[State, QWidget]] = None,
        sx: Optional[Union[State, Dict, Callable, str]] = None,
        timeout: Optional[Union[State, str, int, Dict[str, int]]] = "auto",
        title: Optional[Union[State, str, Callable]] = "Page",
        unmountOnExit: Optional[Union[State, bool]] = None,
        vExpanding: Optional[Union[State, bool]] = None,
        **kwargs
    ):
        super().__init__(**kwargs)

        # Set properties with validation
        self._set_addEndListener(addEndListener)
        self._set_child(child)
        self._set_children(children)
        self._set_classes(classes)
        self._set_collapsedSize(collapsedSize)
        self._set_component(component)
        self._set_easing(easing)
        self._set_id(id)
        self._set_isIn(isIn)
        self._set_maximumHeight(maximumHeight)
        self._set_minimumHeight(minimumHeight)
        self._set_orientation(orientation)
        self._set_showToogleButton(showToogleButton)
        self._set_startActionWidget(startActionWidget)
        self._set_sx(sx)
        self._set_timeout(timeout)
        self._set_title(title)
        self._set_unmountOnExit(unmountOnExit)
        self._set_vExpanding(vExpanding)

        self._animation = None
        
        self._thread_pool = QThreadPool.globalInstance()  # Sử dụng thread pool toàn cục
        
        self._init_ui()


    # Setter and Getter methods for all parameters
    @_validate_param(file_path="qtmui.material.collapse", param_name="addEndListener", supported_signatures=Union[State, Callable, type(None)])
    def _set_addEndListener(self, value):
        self._addEndListener = value

    def _get_addEndListener(self):
        return self._addEndListener.value if isinstance(self._addEndListener, State) else self._addEndListener

    @_validate_param(file_path="qtmui.material.collapse", param_name="child", supported_signatures=Union[State, QWidget, type(None)])
    def _set_child(self, value):
        self._child = value

    def _get_child(self):
        return self._child.value if isinstance(self._child, State) else self._child

    # @_validate_param(file_path="qtmui.material.collapse", param_name="children", supported_signatures=Union[State, QWidget, List[QWidget], type(None)])
    def _set_children(self, value):
        if isinstance(value, list) and not all(isinstance(item, QWidget) for item in value):
            raise ValueError("children must be a QWidget or a list of QWidgets")
        self._children = value

    def _get_children(self):
        return self._children.value if isinstance(self._children, State) else self._children

    @_validate_param(file_path="qtmui.material.collapse", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
    def _set_classes(self, value):
        self._classes = value

    def _get_classes(self):
        return self._classes.value if isinstance(self._classes, State) else self._classes

    @_validate_param(file_path="qtmui.material.collapse", param_name="collapsedSize", supported_signatures=Union[State, int, str])
    def _set_collapsedSize(self, value):
        if isinstance(value, str) and not value.endswith(("px", "%")):
            raise ValueError("collapsedSize as string must end with 'px' or '%'")
        self._collapsedSize = value

    def _get_collapsedSize(self):
        return self._collapsedSize.value if isinstance(self._collapsedSize, State) else self._collapsedSize

    @_validate_param(file_path="qtmui.material.collapse", param_name="component", supported_signatures=Union[State, str, type(None)])
    def _set_component(self, value):
        self._component = value

    def _get_component(self):
        return self._component.value if isinstance(self._component, State) else self._component

    # @_validate_param(file_path="qtmui.material.collapse", param_name="easing", supported_signatures=Union[State, str, Dict[str, str], type(None)])
    def _set_easing(self, value):
        self._easing = value

    def _get_easing(self):
        return self._easing.value if isinstance(self._easing, State) else self._easing

    @_validate_param(file_path="qtmui.material.collapse", param_name="id", supported_signatures=Union[State, str, type(None)])
    def _set_id(self, value):
        self._id = value

    def _get_id(self):
        return self._id.value if isinstance(self._id, State) else self._id

    @_validate_param(file_path="qtmui.material.collapse", param_name="isIn", supported_signatures=Union[State, bool, type(None)])
    def _set_isIn(self, value):
        self._isIn = value

    def _get_isIn(self):
        return self._isIn.value if isinstance(self._isIn, State) else self._isIn

    @_validate_param(file_path="qtmui.material.collapse", param_name="maximumHeight", supported_signatures=Union[State, int, type(None)])
    def _set_maximumHeight(self, value):
        self._maximumHeight = value

    def _get_maximumHeight(self):
        return self._maximumHeight.value if isinstance(self._maximumHeight, State) else self._maximumHeight

    @_validate_param(file_path="qtmui.material.collapse", param_name="minimumHeight", supported_signatures=Union[State, int, type(None)])
    def _set_minimumHeight(self, value):
        self._minimumHeight = value

    def _get_minimumHeight(self):
        return self._minimumHeight.value if isinstance(self._minimumHeight, State) else self._minimumHeight

    @_validate_param(file_path="qtmui.material.collapse", param_name="orientation", supported_signatures=Union[State, str], valid_values=VALID_ORIENTATIONS)
    def _set_orientation(self, value):
        self._orientation = value

    def _get_orientation(self):
        return self._orientation.value if isinstance(self._orientation, State) else self._orientation

    @_validate_param(file_path="qtmui.material.collapse", param_name="showToogleButton", supported_signatures=Union[State, bool])
    def _set_showToogleButton(self, value):
        self._showToogleButton = value

    def _get_showToogleButton(self):
        return self._showToogleButton.value if isinstance(self._showToogleButton, State) else self._showToogleButton

    @_validate_param(file_path="qtmui.material.collapse", param_name="startActionWidget", supported_signatures=Union[State, QWidget, type(None)])
    def _set_startActionWidget(self, value):
        self._startActionWidget = value

    def _get_startActionWidget(self):
        return self._startActionWidget.value if isinstance(self._startActionWidget, State) else self._startActionWidget

    @_validate_param(file_path="qtmui.material.collapse", param_name="sx", supported_signatures=Union[State, Dict, Callable, str, type(None)])
    def _set_sx(self, value):
        self._sx = value

    def _get_sx(self):
        return self._sx.value if isinstance(self._sx, State) else self._sx

    @_validate_param(file_path="qtmui.material.collapse", param_name="timeout", supported_signatures=Union[State, str, int, Dict[str, int], type(None)])
    def _set_timeout(self, value):
        if isinstance(value, dict) and not all(k in ["appear", "enter", "exit"] for k in value):
            raise ValueError("timeout dict must contain 'appear', 'enter', or 'exit' keys")
        self._timeout = value

    def _get_timeout(self):
        return self._timeout.value if isinstance(self._timeout, State) else self._timeout

    @_validate_param(file_path="qtmui.material.collapse", param_name="title", supported_signatures=Union[State, str, Callable])
    def _set_title(self, value):
        self._title = value

    def _get_title(self):
        return self._title.value if isinstance(self._title, State) else self._title

    @_validate_param(file_path="qtmui.material.collapse", param_name="unmountOnExit", supported_signatures=Union[State, bool, type(None)])
    def _set_unmountOnExit(self, value):
        self._unmountOnExit = value

    def _get_unmountOnExit(self):
        return self._unmountOnExit.value if isinstance(self._unmountOnExit, State) else self._unmountOnExit

    @_validate_param(file_path="qtmui.material.collapse", param_name="vExpanding", supported_signatures=Union[State, bool, type(None)])
    def _set_vExpanding(self, value):
        self._vExpanding = value

    def _get_vExpanding(self):
        return self._vExpanding.value if isinstance(self._vExpanding, State) else self._vExpanding


    def _init_ui(self):
        if self._id is not None:
            self.setObjectName(self._id)
        else:
            self.setObjectName(str(uuid.uuid4()))

        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)

        self.btn_toogle = QPushButton()
        self.btn_toogle.setMinimumHeight(40)

        if self._minimumHeight is not None:
            self.setMinimumHeight(self._minimumHeight)

        if self._maximumHeight is not None:
            self.setMaximumHeight(self._maximumHeight)


        self.btn_toogle.setLayout(QHBoxLayout())
        self.btn_toogle.layout().setSpacing(0)

        if self._startActionWidget is not None:
            self.btn_toogle.layout().addWidget(self._startActionWidget)

        self.btn_toogle.layout().addWidget(Typography(text=self._title, variant="h6"))
        self.btn_toogle.layout().addWidget(HSpacer())
        self.btn_icon = QPushButton()
        # self.btn_icon.setIcon(FluentIconBase().icon_(path=":/IconLight/resources/IconLight/Arrow - Down 2.svg", color=self._theme.grey.grey_500))
        self.btn_toogle.layout().addWidget(self.btn_icon)
        self.layout().addWidget(self.btn_toogle)

        if self._children is not None:
            for child in self._children:
                self.layout().addWidget(child)
                # self._add_child_fast(child)
        elif isinstance(self._child, QWidget):
            self.layout().addWidget(self._child)
            # self._add_child_fast(self._child)
            

        if not self._showToogleButton:
            self.btn_toogle.hide()

        self.btn_toogle.clicked.connect(self.set_visible)

        if self._vExpanding:
            self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Maximum)

        # self.is_in_signal.emit(self.__isIn)

        if isinstance(self._isIn, State):
            self._isIn.valueChanged.connect(self.set_visible)
            # self.__isIn = isIn.data
            self.__isIn = self._isIn.value
            self.set_visible(self.__isIn)
        else:
            self.__isIn = self._isIn
            self.is_in_signal.connect(self.set_visible)
            self.is_in_signal.emit(self.__isIn)

        theme = useTheme()
        useEffect(
            self._set_stylesheet,
            [theme.state]
        )
        self._set_stylesheet()

    def _add_child_fast(self, widget):
        worker = WidgetSetter()
        self._thread_pool.start(QTimer.singleShot(0, lambda widget=widget: asyncio.ensure_future(worker.run(self, widget))))  # Chạy setIndexWidget trong thread riêng
     

    def _set_stylesheet(self):
        theme = useTheme()
        self.btn_toogle.setStyleSheet(f"background-color: {theme.palette.background.main};border: 1px solid transparent;border-radius: 5px;")
        self.btn_icon.setStyleSheet("background-color: transparent; border: none;")


    @Slot(bool)
    def set_visible(self, isIn=None):
        if not isIn:
            self.btn_icon.setIcon(PyIconify(key=QTMUI_ASSETS.ICONS.ARROW_DOWN))
            if self._child is not None:
                self._child.hide()
            if self._children is not None:
                for widget in self._children:
                    widget.hide()
        else:
            self.btn_icon.setIcon(PyIconify(key=QTMUI_ASSETS.ICONS.ARROW_UP))
            if self._child is not None:
                self._child.show()
            if self._children is not None:
                for widget in self._children:
                    widget.show()




from typing import TYPE_CHECKING, Callable, Union, Optional
from PySide6.QtWidgets import QFrame, QHBoxLayout, QWidget
from PySide6.QtCore import Qt, Signal

from ...typography import Typography
from ...avatar import Avatar
from ...label import Label
from qtmui.hooks import useState, State

if TYPE_CHECKING:
    from .table import TableWidget


class TableViewCell(QFrame):
    indexRow = Signal(int)
    def __init__(
                self,
                key: str = None,
                data: object = None,
                padding: str = "checkbox",
                align: str = "left",
                children: object = None,
                colSpan: int = None,
                onClick: Callable=None,
                sx: Optional[Union[dict, State]] = None,
                text: str = None
                ):
        super().__init__()
        self.data = data

        self._align = align
        self._children = children
        self._text = text
        self._sx = sx

        self._indexRow = None

        # self.setAttribute(Qt.WA_TransparentForMouseEvents)
        self.setLayout(QHBoxLayout())
        self.layout().setContentsMargins(5,5,5,5)
        
        if self._align == "left":
            self.layout().setAlignment(Qt.AlignmentFlag.AlignLeft)
        elif self._align == "right":
            self.layout().setAlignment(Qt.AlignmentFlag.AlignRight)
        else:
            self.layout().setAlignment(Qt.AlignmentFlag.AlignCenter)
            

        if self._text:
                self.layout().addWidget(Typography(text=self._text, variant="body2"))

        elif self._children:
            if isinstance(self._children, list):
                for widget in self._children:
                    if widget is not None:
                        self.layout().addWidget(widget)
            elif isinstance(self._children, QWidget):
                self.layout().addWidget(self._children)
            elif isinstance(self._children, str):
                cell_widget = Typography(text=self._children, variant="body2", align=self._align)
                self.layout().addWidget(cell_widget)

    
    def enterEvent(self, event):
        try:
            # self.parent().parent()._setHoverRow(self._indexRow)
            # self.parent()._setHoverRow(self._indexRow)
            self.indexRow.emit(self._indexRow)
        except Exception as e:
            print('eeeeeeeee', e)
            pass
        # return super().enterEvent(event)


from typing import Any, Callable, Dict, Optional, Union

from PySide6.QtWidgets import QFrame, QVBoxLayout

from qtmui.hooks import State
from ..textfield import TextField
from ..chip import Chip

class Select(QFrame):
    """
    A select component, styled like Material-UI Select.

    The `Select` component allows users to choose one or more options from a dropdown menu. It integrates
    with the `qtmui` framework, retaining existing parameters, adding new parameters, and aligning with
    MUI Select props. Inherits from OutlinedInput props.

    Parameters
    ----------

    disabled : State or bool, optional
        If True, disables the component. Default is False.
        Can be a `State` object for dynamic updates.

    **kwargs
        Additional keyword arguments passed to the parent `QWidget`,
        supporting props of OutlinedInput.

    Signals
    -------
    onOpen : Signal
        Emitted when the popup opens.
    onClose : Signal
        Emitted when the popup closes.
    changed : Signal
        Emitted when the value changes.
    setupUi : Signal
        Emitted when UI setup is complete.

    Notes
    -----
    - Existing parameters (53) are retained; 9 new parameters added to align with MUI.
    - Props of OutlinedInput are supported via `inputProps` and `**kwargs`.
    - MUI classes applied: `MuiSelect-root`, `Mui-disabled`.
    - Integrates with `TextField` for input rendering.

    Demos:
    - Select: https://qtmui.com/material-ui/qtmui-select/

    API Reference:
    - Select API: https://qtmui.com/material-ui/api/select/
    """
    def __init__(
        self,
        value: Optional[Union[State, Any]] = None,
        onChange: Optional[Union[State, Callable]] = None,
        # labelId: Optional[Union[State, str]] = None,
        # id: Optional[Union[State, str]] = None,
        label: Optional[Union[State, str]] = None,
        fullWidth: Optional[Union[State, bool]] = False,
        multiple: Optional[Union[State, bool]] = False,
        # native: Optional[Union[State, bool]] = False,
        # autoWidth: Optional[Union[State, bool]] = False,
        defaultValue: Optional[Union[State, object]] = None,
        displayEmpty: Optional[Union[State, bool]] = False,
        # input: Optional[Union[State, Any]] = None,
        inputProps: Optional[Union[State, Dict]] = None,
        renderValue: Optional[Union[State, Callable]] = None,
        # MenuProps: Optional[Union[State, Dict]] = None,
        getOptionLabel: Optional[Union[State, Callable]] = None, # mui react không có prop này, nhưng qtmui cần để render các option
        options: Optional[Union[State, object]] = None, # mui react không có prop này, nhưng qtmui cần để render các option
        open: Optional[Union[State, bool]] = False,
        onOpen: Optional[Union[State, Callable]] = None,
        onClose: Optional[Union[State, Callable]] = None,
        defaultOpen: Optional[Union[State, bool]] = False,
        disabled: Optional[Union[State, bool]] = False,
        error: Optional[Union[State, bool]] = False,
        renderOption: Optional[Union[State, Callable]] = None,
        required: Optional[Union[State, bool]] = False,
        readOnly: Optional[Union[State, bool]] = False,
        sx: Optional[Union[State, Dict, Callable, str]] = None,
        size: Optional[Union[State, str]] = "medium", # mui react không có prop này
        *args, **kwargs
    ):
        super().__init__(*args, **kwargs)

        # Lưu các prop
        self._value = value
        self._onChange = onChange
        # self._labelId = labelId
        # self._id = id
        self._label = label
        self._fullWidth = fullWidth
        self._multiple = multiple
        # self._native = native
        # self._autoWidth = autoWidth
        self._defaultValue = defaultValue
        self._displayEmpty = displayEmpty
        # self._input = input
        self._inputProps = inputProps or {}
        self._renderValue = renderValue
        # self._MenuProps = MenuProps or {}
        self._getOptionLabel = getOptionLabel # mui react không có prop này
        self._options = options or [] # mui react không có prop này
        self._open = open
        self._onOpen = onOpen
        self._onClose = onClose
        self._defaultOpen = defaultOpen
        self._disabled = disabled
        self._error = error
        self._renderOption = renderOption # mui react không có prop này
        self._required = required
        self._readOnly = readOnly
        self._sx = sx
        self._size = size  # mui react không có prop này
        
        self._renderTags = None
        
        if self._multiple:
            self._renderTags = lambda selected, getTagProps: (
                [
                    Chip(
                        **getTagProps(index),
                        key=self._getOptionLabel(option),
                        label=self._getOptionLabel(option),
                        size="small",
                        variant="soft",
                        # onDelete=lambda checked, title = option["title"]: self._on_delete_tag(title)
                    )
                    for index, option in selected.items()
                ]
            )

        self._init_ui()


    def _init_ui(self):
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)

        input_params = {
            "isSlectComponent": True,
            "select": True,
            "value": self._value,
            "onChange": self._onChange,
            # "labelId": self._labelId,
            # "id": self._id,
            "label": self._label,
            "fullWidth": self._fullWidth,
            "multiple": self._multiple,
            # "native": self._native,
            # "autoWidth": self._autoWidth,
            "defaultValue": self._defaultValue,
            # "displayEmpty": self._displayEmpty,
            # "input": self._input,
            "inputProps": self._inputProps,
            "renderValue": self._renderValue,
            # "MenuProps": self._MenuProps,
            "getOptionLabel": self._getOptionLabel,
            "options": self._options,
            "open": self._open,
            "onOpen": self._onOpen,
            "onClose": self._onClose,
            "defaultOpen": self._defaultOpen,
            "disabled": self._disabled,
            "error": self._error,
            "renderOption": self._renderOption,
            "required": self._required,
            "readOnly": self._readOnly,
            "sx": self._sx,
            "size": self._size,
            "renderTags": self._renderTags,
        }

        self._inputField = TextField(**input_params)
        self.layout().addWidget(self._inputField)






# from typing import Optional, Union, Callable, Any, Dict, List
# import uuid

# from PySide6.QtWidgets import QApplication, QPushButton, QVBoxLayout, QWidget, QSizePolicy
# from PySide6.QtGui import QPainter, QColor, QBrush, QLinearGradient, QPainterPath
# from PySide6.QtCore import QPropertyAnimation, QObject, Qt, QRect, Property, QEasingCurve
# from ..widget_base import PyWidgetBase
# from qtmui.material.styles import useTheme
# from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
# from qtmui.hooks import State
# from ..utils.validate_params import _validate_param

# class RippleEffect(QObject):
#     """
#     A helper class to manage ripple animation effects for Skeleton.

#     Parameters
#     ----------
#     parent : QObject, optional
#         The parent object. Default is None.
#     """
#     def __init__(self, parent=None):
#         super().__init__(parent)
#         self._rectX1 = 0
#         self._rectY1 = 0
#         self._rectWidth = 100
#         self._rectHeight = 40
#         self._max_rectX1 = 100
#         self._opacity = 1.0
#         self._brush_color = QColor(227, 227, 227)

#     def rectX1(self):
#         return self._rectX1

#     def setRectX1(self, rectX1):
#         self._rectX1 = rectX1
#         self.parent().update()

#     def opacity(self):
#         return self._opacity

#     def setOpacity(self, opacity):
#         self._opacity = opacity
#         self.parent().update()

#     def brush_color(self):
#         return self._brush_color

#     def setBrushColor(self, brush_color):
#         self._brush_color = brush_color
#         self.parent().update()

#     rectX1 = Property(int, fget=rectX1, fset=setRectX1)
#     opacity = Property(float, fget=opacity, fset=setOpacity)
#     brush_color = Property(QColor, fget=brush_color, fset=setBrushColor)

# class Skeleton(PyWidgetBase):
#     """
#     A skeleton component, styled like Material-UI Skeleton.

#     The `Skeleton` component is a placeholder used to indicate loading content. It supports
#     animations and variants, aligning with MUI Skeleton props. Inherits from native component props.

#     Parameters
#     ----------
#     animation : State, str, or bool, optional
#         The animation type ('pulse', 'wave', False). Default is 'pulse'.
#         Can be a `State` object for dynamic updates.
#     children : State or QWidget, optional
#         Optional children to infer width and height from. Default is None.
#         Can be a `State` object for dynamic updates.
#     classes : State or Dict, optional
#         Override or extend styles applied to the component. Default is None.
#         Can be a `State` object for dynamic updates.
#     component : State or type, optional
#         The component used for the root node. Default is None (uses QPushButton).
#         Can be a `State` object for dynamic updates.
#     height : State, int, or str, optional
#         Height of the skeleton. Default is None.
#         Can be a `State` object for dynamic updates.
#     sx : State, List, Dict, Callable, or None, optional
#         System prop for CSS overrides. Default is None.
#         Can be a `State` object for dynamic updates.
#     variant : State or str, optional
#         Type of skeleton ('circular', 'rectangular', 'rounded', 'text'). Default is None.
#         Can be a `State` object for dynamic updates.
#     width : State, int, or str, optional
#         Width of the skeleton. Default is 100.
#         Can be a `State` object for dynamic updates.
#     key : State or str, optional
#         Key for the component. Default is None.
#         Can be a `State` object for dynamic updates.
#     color : State or str, optional
#         Background color of the skeleton. Default is '#ededed'.
#         Can be a `State` object for dynamic updates.
#     fullwidth : State or bool, optional
#         If True, skeleton takes full width. Default is False.
#         Can be a `State` object for dynamic updates.
#     custom_radius : State, int, or None, optional
#         Custom radius for rounded corners. Default is None.
#         Can be a `State` object for dynamic updates.
#     duration : State or int, optional
#         Duration of the animation in milliseconds. Default is 1500.
#         Can be a `State` object for dynamic updates.
#     startColorAt : State or QColor, optional
#         Starting color for pulse animation. Default is QColor(227, 227, 227).
#         Can be a `State` object for dynamic updates.
#     endColorAt : State or QColor, optional
#         Ending color for pulse animation. Default is QColor(217, 217, 217).
#         Can be a `State` object for dynamic updates.
#     **kwargs
#         Additional keyword arguments passed to the parent component,
#         supporting native component props.

#     Notes
#     -----
#     - Existing parameters (14) are retained and aligned with MUI Skeleton.
#     - MUI classes applied: `MuiSkeleton-root`.
#     - Integrates with `RippleEffect` for animations.

#     Demos:
#     - Skeleton: https://qtmui.com/material-ui/qtmui-skeleton/

#     API Reference:
#     - Skeleton API: https://qtmui.com/material-ui/api/skeleton/
#     """

#     VALID_ANIMATIONS = ['pulse', 'wave', False]
#     VALID_VARIANTS = ['circular', 'rectangular', 'rounded', 'text']

#     def __init__(
#         self,
#         animation: Union[State, str, bool] = "pulse",
#         children: Optional[Union[State, QWidget]] = None,
#         classes: Optional[Union[State, Dict]] = None,
#         component: Optional[Union[State, type]] = None,
#         height: Optional[Union[State, int, str]] = None,
#         sx: Optional[Union[State, List, Dict, Callable]] = None,
#         variant: Optional[Union[State, str]] = None,
#         width: Optional[Union[State, int, str]] = 100,
#         key: Optional[Union[State, str]] = None,
#         color: Optional[Union[State, str]] = "#ededed",
#         fullwidth: Union[State, bool] = False,
#         custom_radius: Optional[Union[State, int]] = None,
#         duration: Union[State, int] = 1500,
#         startColorAt: Union[State, QColor] = QColor(227, 227, 227),
#         endColorAt: Union[State, QColor] = QColor(217, 217, 217),
#         *args, **kwargs
#     ):
#         # Use component if provided, else default to QPushButton
#         root_component = (component.value if isinstance(component, State) else component) or QPushButton
#         super().__init__(*args, **kwargs)
#         self.__class__ = type('Skeleton', (root_component, PyWidgetBase), {})
#         self.theme = useTheme()
#         self._widget_references = []

#         # Set properties with validation
#         self._set_animation(animation)
#         self._set_children(children)
#         self._set_classes(classes)
#         self._set_component(component)
#         self._set_height(height)
#         self._set_sx(sx)
#         self._set_variant(variant)
#         self._set_width(width)
#         self._set_key(key)
#         self._set_color(color)
#         self._set_fullwidth(fullwidth)
#         self._set_custom_radius(custom_radius)
#         self._set_duration(duration)
#         self._set_startColorAt(startColorAt)
#         self._set_endColorAt(endColorAt)

#         self._init_ui()
#         self.slot_set_stylesheet()
#         self.theme.state.valueChanged.connect(self.slot_set_stylesheet)
#         self.destroyed.connect(self._on_destroyed)
#         self._connect_signals()

#     # Setter and Getter methods
#     @_validate_param(file_path="qtmui.material.skeleton", param_name="animation", supported_signatures=Union[State, str, bool], valid_values=VALID_ANIMATIONS)
#     def _set_animation(self, value):
#         """Assign value to animation."""
#         self._animation = value
#         if hasattr(self, 'ripple_effect'):
#             self.set_animation_mode(self._get_animation())

#     def _get_animation(self):
#         """Get the animation value."""
#         return self._animation.value if isinstance(self._animation, State) else self._animation

#     @_validate_param(file_path="qtmui.material.skeleton", param_name="children", supported_signatures=Union[State, QWidget, type(None)])
#     def _set_children(self, value):
#         """Assign value to children."""
#         self._children = value
#         if isinstance(value, QWidget):
#             self._widget_references.append(value)
#         elif isinstance(value, State) and isinstance(value.value, QWidget):
#             self._widget_references.append(value.value)

#     def _get_children(self):
#         """Get the children value."""
#         return self._children.value if isinstance(self._children, State) else self._children

#     @_validate_param(file_path="qtmui.material.skeleton", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
#     def _set_classes(self, value):
#         """Assign value to classes."""
#         self._classes = value

#     def _get_classes(self):
#         """Get the classes value."""
#         return self._classes.value if isinstance(self._classes, State) else self._classes

#     @_validate_param(file_path="qtmui.material.skeleton", param_name="component", supported_signatures=Union[State, type, type(None)])
#     def _set_component(self, value):
#         """Assign value to component."""
#         self._component = value

#     def _get_component(self):
#         """Get the component value."""
#         return self._component.value if isinstance(self._component, State) else self._component

#     @_validate_param(file_path="qtmui.material.skeleton", param_name="height", supported_signatures=Union[State, int, str, type(None)])
#     def _set_height(self, value):
#         """Assign value to height."""
#         self._height = value

#     def _get_height(self):
#         """Get the height value."""
#         return self._height.value if isinstance(self._height, State) else self._height

#     @_validate_param(file_path="qtmui.material.skeleton", param_name="sx", supported_signatures=Union[State, List, Dict, Callable, type(None)])
#     def _set_sx(self, value):
#         """Assign value to sx."""
#         self._sx = value

#     def _get_sx(self):
#         """Get the sx value."""
#         return self._sx.value if isinstance(self._sx, State) else self._sx

#     @_validate_param(file_path="qtmui.material.skeleton", param_name="variant", supported_signatures=Union[State, str, type(None)], valid_values=VALID_VARIANTS)
#     def _set_variant(self, value):
#         """Assign value to variant."""
#         self._variant = value

#     def _get_variant(self):
#         """Get the variant value."""
#         return self._variant.value if isinstance(self._variant, State) else self._variant

#     @_validate_param(file_path="qtmui.material.skeleton", param_name="width", supported_signatures=Union[State, int, str, type(None)])
#     def _set_width(self, value):
#         """Assign value to width."""
#         self._width = value

#     def _get_width(self):
#         """Get the width value."""
#         return self._width.value if isinstance(self._width, State) else self._width

#     @_validate_param(file_path="qtmui.material.skeleton", param_name="key", supported_signatures=Union[State, str, type(None)])
#     def _set_key(self, value):
#         """Assign value to key."""
#         self._key = value

#     def _get_key(self):
#         """Get the key value."""
#         return self._key.value if isinstance(self._key, State) else self._key

#     @_validate_param(file_path="qtmui.material.skeleton", param_name="color", supported_signatures=Union[State, str, type(None)])
#     def _set_color(self, value):
#         """Assign value to color."""
#         self._color = value

#     def _get_color(self):
#         """Get the color value."""
#         return self._color.value if isinstance(self._color, State) else self._color

#     @_validate_param(file_path="qtmui.material.skeleton", param_name="fullwidth", supported_signatures=Union[State, bool])
#     def _set_fullwidth(self, value):
#         """Assign value to fullwidth."""
#         self._fullwidth = value

#     def _get_fullwidth(self):
#         """Get the fullwidth value."""
#         return self._fullwidth.value if isinstance(self._fullwidth, State) else self._fullwidth

#     @_validate_param(file_path="qtmui.material.skeleton", param_name="custom_radius", supported_signatures=Union[State, int, type(None)])
#     def _set_custom_radius(self, value):
#         """Assign value to custom_radius."""
#         self._custom_radius = value

#     def _get_custom_radius(self):
#         """Get the custom_radius value."""
#         return self._custom_radius.value if isinstance(self._custom_radius, State) else self._custom_radius

#     @_validate_param(file_path="qtmui.material.skeleton", param_name="duration", supported_signatures=Union[State, int])
#     def _set_duration(self, value):
#         """Assign value to duration."""
#         self._duration = value

#     def _get_duration(self):
#         """Get the duration value."""
#         return self._duration.value if isinstance(self._duration, State) else self._duration

#     @_validate_param(file_path="qtmui.material.skeleton", param_name="startColorAt", supported_signatures=Union[State, QColor])
#     def _set_startColorAt(self, value):
#         """Assign value to startColorAt."""
#         self._startColorAt = value

#     def _get_startColorAt(self):
#         """Get the startColorAt value."""
#         return self._startColorAt.value if isinstance(self._startColorAt, State) else self._startColorAt

#     @_validate_param(file_path="qtmui.material.skeleton", param_name="endColorAt", supported_signatures=Union[State, QColor])
#     def _set_endColorAt(self, value):
#         """Assign value to endColorAt."""
#         self._endColorAt = value

#     def _get_endColorAt(self):
#         """Get the endColorAt value."""
#         return self._endColorAt.value if isinstance(self._endColorAt, State) else self._endColorAt

#     def _init_ui(self):
#         """Initialize the UI based on props."""
#         self.setObjectName(str(uuid.uuid4()))

#         # Infer size from children if not provided
#         children = self._get_children()
#         if children and isinstance(children, QWidget) and (not self._get_width() or not self._get_height()):
#             size = children.sizeHint()
#             self._width = self._width or size.width()
#             self._height = self._height or size.height()

#         # Set variant-specific properties
#         if self._get_variant():
#             if self._get_variant() == "circular":
#                 self._width = self._height = self._get_width() or self._get_height() or 40
#                 self._radius = self._get_custom_radius() if self._get_custom_radius() is not None else self._height / 2
#                 self.setFixedSize(self._width, self._height)
#             elif self._get_variant() == "text":
#                 self._radius = self._get_custom_radius() if self._get_custom_radius() is not None else 4
#                 self._height = self._get_height() or 24
#             elif self._get_variant() == "rounded":
#                 self._radius = self._get_custom_radius() if self._get_custom_radius() is not None else 4
#                 self._height = self._get_height() or 24
#             elif self._get_variant() == "rectangular":
#                 self._radius = self._get_custom_radius() if self._get_custom_radius() is not None else 0
#                 self._height = self._get_height() or 60
#             if not self._get_width() and self._get_fullwidth():
#                 self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
#                 self.setMinimumHeight(self._height)
#             else:
#                 self.setFixedSize(self._get_width() or 100, self._height)
#         else:
#             self._radius = self._get_custom_radius() if self._get_custom_radius() is not None else (self._get_height() or 24) / 2
#             self._height = self._get_height() or 24
#             self._width = self._get_width() or 100
#             if self._get_fullwidth():
#                 self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
#                 self.setMinimumHeight(self._height)
#             else:
#                 self.setFixedSize(self._width, self._height)

#         # Initialize ripple effect and animations
#         self.ripple_effect = RippleEffect(self)
#         self.animation_rectX1 = QPropertyAnimation(self.ripple_effect, b"rectX1", self)
#         self.opacity_animation = QPropertyAnimation(self.ripple_effect, b"opacity", self)
#         self.color_animation = QPropertyAnimation(self.ripple_effect, b"brush_color", self)

#         # Set animation mode
#         self.set_animation_mode(self._get_animation())

#     def slot_set_stylesheet(self, value=None):
#         """Set the stylesheet for the Skeleton."""
#         self._set_stylesheet()

#     def _set_stylesheet(self, component_styled=None):
#         """Apply stylesheet based on theme and props."""
#         self.theme = useTheme()
#         component_styled = component_styled or self.theme.components
#         skeleton_styles = component_styled.get("Skeleton", {}).get("styles", {})
#         root_styles = skeleton_styles.get("root", {})
#         root_qss = get_qss_style(root_styles)

#         # Handle sx
#         sx = self._get_sx()
#         sx_qss = ""
#         if sx:
#             if isinstance(sx, (list, dict)):
#                 sx_qss = get_qss_style(sx, class_name=f"#{self.objectName()}")
#             elif isinstance(sx, Callable):
#                 sx_result = sx()
#                 if isinstance(sx_result, (list, dict)):
#                     sx_qss = get_qss_style(sx_result, class_name=f"#{self.objectName()}")
#                 elif isinstance(sx_result, str):
#                     sx_qss = sx_result
#             elif isinstance(sx, str) and sx != "":
#                 sx_qss = sx

#         # Handle classes
#         classes = self._get_classes()
#         classes_qss = get_qss_style(classes, class_name=f"#{self.objectName()}") if classes else ""

#         # Apply MUI classes
#         mui_classes = ["MuiSkeleton-root"]

#         stylesheet = f"""
#             #{self.objectName()} {{
#                 {root_qss}
#                 {classes_qss}
#                 background: {self._get_color() or '#ededed'};
#             }}
#             {sx_qss}
#         """
#         self.setStyleSheet(stylesheet)

#     def set_animation_mode(self, animation):
#         """Set the animation mode for the skeleton."""
#         if animation == "wave":
#             self.animation_rectX1.setEasingCurve(QEasingCurve.Linear)
#             self.opacity_animation.setEasingCurve(QEasingCurve.Linear)

#             self.ripple_effect._max_rectX1 = self.width()
#             self.ripple_effect.opacity = 0.3
#             self.animation_rectX1.setStartValue(-self.ripple_effect._rectWidth)
#             self.animation_rectX1.setEndValue(self.width())
#             self.animation_rectX1.setDuration(self._get_duration())

#             self.opacity_animation.setStartValue(0.7)
#             self.opacity_animation.setEndValue(0.0)
#             self.opacity_animation.setDuration(self._get_duration())

#             self.animation_rectX1.setLoopCount(-1)
#             self.opacity_animation.setLoopCount(-1)

#             self.animation_rectX1.start()
#             self.opacity_animation.start()

#         elif animation == "pulse":
#             self.color_animation.setKeyValueAt(0.0, self._get_startColorAt())
#             self.color_animation.setKeyValueAt(0.5, self._get_endColorAt())
#             self.color_animation.setKeyValueAt(1.0, self._get_startColorAt())

#             self.color_animation.setEasingCurve(QEasingCurve.InOutQuad)
#             self.color_animation.setDuration(self._get_duration())
#             self.color_animation.setLoopCount(-1)
#             self.color_animation.start()

#         elif animation is False:
#             self.animation_rectX1.stop()
#             self.opacity_animation.stop()
#             self.color_animation.stop()
#             self.ripple_effect.setBrushColor(QColor(227, 227, 227))

#     def paintEvent(self, event):
#         """Custom paint event for rendering skeleton with animation."""
#         super().paintEvent(event)
#         if self._get_animation() == "pulse" and self.ripple_effect.brush_color is not None:
#             painter = QPainter(self)
#             painter.setRenderHint(QPainter.Antialiasing, True)
#             painter.setPen(Qt.NoPen)

#             clip_path = QPainterPath()
#             clip_path.addRoundedRect(0, 0, self.width(), self.height(), self._radius, self._radius)
#             painter.setClipPath(clip_path)

#             gradient = QLinearGradient(0, 0, self.width(), 0)
#             gradient.setColorAt(0.0, self.ripple_effect.brush_color)
#             gradient.setColorAt(1.0, self.ripple_effect.brush_color)

#             painter.setBrush(QBrush(gradient))
#             rect = QRect(0, 0, self.width(), self.height())
#             painter.drawRoundedRect(rect, self._radius, self._radius)

#             painter.end()

#         elif self._get_animation() == "wave" and self.ripple_effect.opacity > 0:
#             painter = QPainter(self)
#             painter.setRenderHint(QPainter.Antialiasing, True)
#             painter.setPen(Qt.NoPen)

#             clip_path = QPainterPath()
#             clip_path.addRoundedRect(0, 0, self.width(), self.height(), self._radius, self._radius)
#             painter.setClipPath(clip_path)

#             gradient = QLinearGradient(self.ripple_effect.rectX1, 0,
#                                       self.ripple_effect.rectX1 + self.ripple_effect._rectWidth, 0)
#             gradient.setColorAt(0.0, QColor(227, 227, 227))
#             gradient.setColorAt(0.5, QColor(217, 217, 217))
#             gradient.setColorAt(1.0, QColor(227, 227, 227))

#             painter.setBrush(QBrush(gradient))
#             path = QPainterPath()
#             rect = QRect(self.ripple_effect.rectX1, 0, self.ripple_effect._rectWidth, self.height())
#             path.addRoundedRect(rect, self._radius, self._radius)
#             painter.drawPath(path)

#             painter.end()

#     def _connect_signals(self):
#         """Connect valueChanged signals of State parameters to their slots."""
#         if isinstance(self._animation, State):
#             self._animation.valueChanged.connect(self._on_animation_changed)
#         if isinstance(self._children, State):
#             self._children.valueChanged.connect(self._on_children_changed)
#         if isinstance(self._classes, State):
#             self._classes.valueChanged.connect(self.slot_set_stylesheet)
#         if isinstance(self._height, State):
#             self._height.valueChanged.connect(self._on_size_changed)
#         if isinstance(self._sx, State):
#             self._sx.valueChanged.connect(self.slot_set_stylesheet)
#         if isinstance(self._variant, State):
#             self._variant.valueChanged.connect(self._on_variant_changed)
#         if isinstance(self._width, State):
#             self._width.valueChanged.connect(self._on_size_changed)
#         if isinstance(self._color, State):
#             self._color.valueChanged.connect(self.slot_set_stylesheet)
#         if isinstance(self._fullwidth, State):
#             self._fullwidth.valueChanged.connect(self._on_size_changed)
#         if isinstance(self._custom_radius, State):
#             self._custom_radius.valueChanged.connect(self._on_variant_changed)
#         if isinstance(self._duration, State):
#             self._duration.valueChanged.connect(self._on_duration_changed)
#         if isinstance(self._startColorAt, State):
#             self._startColorAt.valueChanged.connect(self._on_colors_changed)
#         if isinstance(self._endColorAt, State):
#             self._endColorAt.valueChanged.connect(self._on_colors_changed)

#     def _on_animation_changed(self):
#         """Handle animation change."""
#         self.set_animation_mode(self._get_animation())

#     def _on_children_changed(self):
#         """Handle children change."""
#         self._init_ui()

#     def _on_size_changed(self):
#         """Handle size change."""
#         self._init_ui()

#     def _on_variant_changed(self):
#         """Handle variant change."""
#         self._init_ui()

#     def _on_duration_changed(self):
#         """Handle duration change."""
#         self.set_animation_mode(self._get_animation())

#     def _on_colors_changed(self):
#         """Handle colors change."""
#         self.set_animation_mode(self._get_animation())

#     def _on_destroyed(self):
#         """Clean up on destruction."""
#         self._widget_references.clear()


from typing import Dict, TYPE_CHECKING
if TYPE_CHECKING:
    from qtmui.material.styles.create_theme.theme_reducer import ThemeState

from .properties_name import *
from ....system.color_manipulator import alpha


def masonry(_theme) -> Dict:
    theme: ThemeState = _theme
    # theme = _theme
    return {
        'PyMasonry': {
            'styles': {
                "root": {

                },
            },
        },
    }



# coding: utf-8
import uuid
from typing import List, Union, Optional, Dict

from PySide6.QtGui import QAction
from PySide6.QtWidgets import (
    QTableWidget, 
    QHeaderView, 
    QWidget, 
    QTableWidgetItem, 
    QStyledItemDelegate, 
    QApplication, 
    QStyleOptionViewItem,
    QTableView, 
    QTableWidget, 
    QWidget, 
    QTableWidgetItem, 
    QStyle,
    QStyleOptionButton, 
    QFrame, 
    QVBoxLayout,
    QProxyStyle,
    QStyleOption,
    QCheckBox,
    QHBoxLayout,
)
from PySide6.QtCore import (
    Qt, 
    QMargins, 
    QModelIndex, 
    QItemSelectionModel, 
    Property, 
    QRectF, 
    QRect,
)
from PySide6.QtGui import QPainter, QColor, QKeyEvent, QPalette, QBrush, QFont

from typing import TYPE_CHECKING, Callable
if TYPE_CHECKING:
    from .table_row import TableRow

from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style

from qtmui.material.styles import useTheme
from ....i18n.use_translation import translate, i18n

from ....common.font import getFont
from ....common.style_sheet import isDarkTheme, FluentStyleSheet, themeColor, setCustomStyleSheet
from ...widgets.check_box import CheckBoxIcon
from ...widgets.line_edit import LineEdit
from ...py_iconify import PyIconify
from ...widgets.scroll_bar import SmoothScrollDelegate
from ...._____assets import ASSETS

from ...checkbox import Checkbox
from ...button import Button
from ...box import Box


class CustomHeaderViewCheckbox(QHeaderView):
    def __init__(self, orientation, parent=None):
        super().__init__(orientation, parent)
        self.theme = useTheme()
        # Màu nền của header
        self._headerBackgroundColor = QColor(self.theme.palette.background.content)
        # Màu của border
        self._borderColor = QColor(0, 0, 0)
        self._borderWidth = 2
        # Thuộc tính trạng thái checkbox (mặc định Off)
        self._checkboxChecked = False

    def _set_theme(self):
        self.theme = useTheme()
        self._headerBackgroundColor = QColor(self.theme.palette.background.content)

    def paintSection(self, painter, rect, logicalIndex):
        painter.save()

        # Vẽ đường viền xung quanh header section
        painter.setPen(self._borderColor)
        painter.drawRect(rect)

        # Nếu là cột thứ nhất, vẽ thêm checkbox trước tiêu đề
        if logicalIndex == 0:
            # Xác định kích thước và vị trí của checkbox
            constMargin = 5
            checkBoxSize = 16
            checkBoxRect = QRect(rect.left() + constMargin,
                                 rect.top() + (rect.height() - checkBoxSize) // 2,
                                 checkBoxSize,
                                 checkBoxSize)

            # Tạo QStyleOptionButton cho checkbox
            option = QStyleOptionButton()
            option.rect = checkBoxRect
            option.state = QStyle.State_Enabled | QStyle.State_Active
            if self._checkboxChecked:
                option.state |= QStyle.State_On
            else:
                option.state |= QStyle.State_Off

            # Vẽ checkbox
            self.style().drawControl(QStyle.CE_CheckBox, option, painter)

            # Điều chỉnh vùng vẽ tiêu đề để không bị che bởi checkbox
            textRect = QRect(checkBoxRect.right() + constMargin,
                             rect.top(),
                             rect.width() - checkBoxRect.width() - 2 * constMargin,
                             rect.height())
        else:
            # Với các cột khác, sử dụng toàn bộ rect cho tiêu đề
            textRect = rect

        # Lấy tiêu đề từ model
        title = self.model().headerData(logicalIndex, self.orientation(), Qt.DisplayRole)
        if title:
            painter.setPen(QColor(self.theme.palette.text.primary))
            font = self.font()
            font.setBold(True)
            painter.setFont(font)
            # Vẽ tiêu đề căn giữa theo chiều dọc và căn trái theo chiều ngang
            painter.drawText(textRect, Qt.AlignLeft | Qt.AlignVCenter, str(title))

        painter.restore()


class CustomHeaderView(QHeaderView):
    def __init__(self, orientation, parent=None):
        super().__init__(orientation, parent)

        self.theme = useTheme()

        # Màu nền của header
        self._headerBackgroundColor = QColor(self.theme.palette.background.content)  # Màu nền
        # Màu của border
        self._borderColor = QColor(0, 0, 0)  # Màu đường viền
        self._borderWidth = 2  # Độ dày của đường viền

    def _set_theme(self):
        self.theme = useTheme()
        self._headerBackgroundColor = QColor(self.theme.palette.background.content)  # Màu nền

    def paintSection(self, painter, rect, logicalIndex):
        painter.save()
        
        # Tô màu nền của header
        # painter.setBrush(self._headerBackgroundColor)
        # painter.fillRect(rect, self._headerBackgroundColor)

        # Vẽ đường viền xung quanh (toàn bộ header)
        # painter.setPen(self._borderColor)
        # painter.drawRect(rect)  # Vẽ đường viền chung cho header

        # Vẽ tiêu đề của cột, căn giữa
        # textRect = rect
        # textRect.setLeft(rect.left() + 5)  # Thêm khoảng cách bên trái để tránh dính vào cạnh
        # textRect.setRight(rect.right() - 5)  # Thêm khoảng cách bên phải
        title = self.model().headerData(logicalIndex, self.orientation(), Qt.DisplayRole)
        if title:
            painter.setPen(QColor(self.theme.palette.text.primary))
            font = self.font()
            font.setBold(True)
            painter.setFont(font)
            # painter.drawText(rect, Qt.AlignLeft | Qt.AlignVCenter, title)
            # painter.drawLine(rect.right(), rect.top(), rect.right(), rect.bottom())  # Border trên phải

        painter.restore()


from typing import Callable
from PySide6.QtWidgets import QFrame, QHBoxLayout, QSizePolicy
from PySide6.QtCore import Signal

from qtmui.hooks import State

from ..checkbox.checkbox import Checkbox


class ListItemCheckbox(QFrame):

    themeChanged = Signal()

    def __init__(self, 
                 alignItems='center',  # Căn chỉnh items
                 autoFocus=False,  # Tự động focus khi mount
                 children=None,  # Nội dung của ListItemButton
                 classes=None,  # Ghi đè hoặc mở rộng các style
                 component=QFrame,  # Thành phần dùng cho root node
                 dense=False,  # Sử dụng padding dọc nhỏ gọn
                 disabled=False,  # Vô hiệu hóa component
                 disableGutters=False,  # Loại bỏ padding trái/phải
                 divider=False,  # Thêm viền dưới
                 focusVisibleClassName=None,  # Lớp CSS khi focus
                 key = None,
                 minHeight=None,
                 onClick: Callable=None,
                 selected=False,  # Sử dụng style khi được chọn
                 sx=None,  # Hệ thống prop cho overrides và styles
                 size="medium",  # Hệ thống prop cho overrides và styles
                 parent=None,  # Parent widget
                 selectedKey: State=None,  # Parent widget
                 **kwargs):
        super().__init__(**kwargs)

        # Gán các props thành thuộc tính của class
        self._alignItems = alignItems
        self._autoFocus = autoFocus
        self._children = children
        self._classes = classes
        self._component = component
        self._dense = dense
        self._disabled = disabled
        self._disableGutters = disableGutters
        self._divider = divider
        self._focusVisibleClassName = focusVisibleClassName
        self._selected = selected
        self._sx = sx

        self._size = size

        self._key = key
        self._onListItemButtonClick = onClick
        self._selectedKey = selectedKey

        # Thiết lập trạng thái ban đầu
        self.setDisabled(self._disabled)
        # self.setAutoDefault(self._autoFocus)

        # Thiết lập căn chỉnh
        # self.setStyleSheet(self._get_align_items_style(self._alignItems))

        # Áp dụng thêm styles từ sx nếu có
        if self._sx:
            self._apply_sx(self._sx)

        self.setLayout(QHBoxLayout())
        # self.layout().setContentsMargins(9,9,9,9)
        self.layout().setContentsMargins(0,0,0,0)
        self.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)

        if minHeight:
            self.setMinimumHeight(minHeight)

        self._checkbox = Checkbox(disableRipple=True, disableGutters=True, size="small", color="primary")
        self.layout().addWidget(self._checkbox)

        if self._children:
            if not isinstance(self._children, list):
                raise TypeError("children must be type (list)")
            
            for child in self._children:
                if hasattr(child, '_secondary'):
                    if getattr(child, '_secondary'):
                        self.setFixedHeight(48)
                self.layout().addWidget(child)


    def _get_align_items_style(self, alignItems):
        """Thiết lập style cho align-items"""
        if alignItems == 'center':
            return "text-align: center;"
        elif alignItems == 'flex-start':
            return "text-align: left;"
        return ""

    def _apply_sx(self, sx):
        """Áp dụng hệ thống overrides cho styles."""
        if isinstance(sx, dict):
            for key, value in sx.items():
                self.setStyleSheet(f"{key}: {value};")
        elif callable(sx):
            sx(self)  # sx có thể là hàm áp dụng style

    def focusInEvent(self, event):
        """Quản lý lớp khi element được focus"""
        super().focusInEvent(event)
        if self._focusVisibleClassName:
            self.setStyleSheet(self.styleSheet() + f" {self._focusVisibleClassName} {{ outline: 1px solid #000; }}")

    def mouseReleaseEvent(self, event):
        if self._onListItemButtonClick:
            self._onListItemButtonClick(self._key)
            
        return super().mouseReleaseEvent(event)



import asyncio
from typing import Callable, Union, Dict, Optional, List
from PySide6.QtWidgets import QHBoxLayout, QVBoxLayout, QFrame, QSizePolicy, QLabel, QWidget
from PySide6.QtCore import Qt, QTimer, QEvent
import uuid
from qtmui.hooks import State
from ..spacer import HSpacer
from ..widget_base import PyWidgetBase
from ...material.styles import useTheme
from ..typography import Typography
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
from ..utils.validate_params import _validate_param

class FormControlLabel(QFrame, PyWidgetBase):
    """
    A component that provides a label for a control element like Radio, Switch, or Checkbox.

    The `FormControlLabel` component is used to associate a label with a form control,
    supporting all props of the Material-UI `FormControlLabel` component, as well as
    additional custom props.

    Parameters
    ----------
    control : State or QWidget, required
        A control element (e.g., Radio, Switch, Checkbox).
        Can be a `State` object for dynamic updates.
    checked : State or bool, optional
        If True, the component appears selected. Default is False.
        Can be a `State` object for dynamic updates.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    componentsProps : State or dict, optional
        Props used for each slot inside (deprecated, use `slotProps` instead). Default is None.
        Can be a `State` object for dynamic updates.
    disabled : State or bool, optional
        If True, the control is disabled. Default is False.
        Can be a `State` object for dynamic updates.
    disableTypography : State or bool, optional
        If True, the label is rendered without a typography node. Default is False.
        Can be a `State` object for dynamic updates.
    inputRef : State or object, optional
        Reference to the input element. Default is None.
        Can be a `State` object for dynamic updates.
    label : State, str, QWidget, List[QWidget], or None, optional
        Text or element used as the label. Default is None.
        Can be a `State` object for dynamic updates.
    labelPlacement : State or str, optional
        Position of the label ("bottom", "end", "start", "top"). Default is "end".
        Can be a `State` object for dynamic updates.
    key : State or str, optional
        Identifier for the component (custom feature, not part of Material-UI). Default is None.
        Can be a `State` object for dynamic updates.
    onChange : State or Callable, optional
        Callback fired when the state changes. Default is None.
        Signature: function(event: dict) => void
        Can be a `State` object for dynamic updates.
    required : State or bool, optional
        If True, the label indicates that the input is required. Default is False.
        Can be a `State` object for dynamic updates.
    slotProps : State or dict, optional
        Props used for each slot inside. Default is None.
        Can be a `State` object for dynamic updates.
    slots : State or dict, optional
        Components used for each slot inside (e.g., typography). Default is None.
        Can be a `State` object for dynamic updates.
    sx : State, list, dict, Callable, str, or None, optional
        System prop for CSS overrides and additional styles. Default is None.
        Can be a `State` object for dynamic updates.
    value : State or any, optional
        Value of the component. Default is None.
        Can be a `State` object for dynamic updates.
    highlight : State or bool, optional
        If True, applies a highlight border (custom feature, not part of Material-UI). Default is False.
        Can be a `State` object for dynamic updates.
    fullWidth : State or bool, optional
        If True, the component takes up the full width (custom feature, not part of Material-UI). Default is False.
        Can be a `State` object for dynamic updates.
    spacing : State or int, optional
        Spacing between elements in the layout (custom feature, not part of Material-UI). Default is 6.
        Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to the parent `QFrame` class, supporting
        props of the native component (e.g., style, className).

    Attributes
    ----------
    VALID_LABEL_PLACEMENTS : list[str]
        Valid values for `labelPlacement`: ["bottom", "end", "start", "top"].

    Notes
    -----
    - Props of the native component are supported via `**kwargs` (e.g., `style`, `className`).
    - The `key`, `highlight`, `fullWidth`, and `spacing` parameters are custom features, not part of Material-UI's `FormControlLabel`.
    - The `componentsProps` prop is deprecated; use `slotProps` instead.
    - The `control` prop is required and must be a `QWidget` (e.g., Radio, Switch, Checkbox).

    Demos:
    - FormControlLabel: https://qtmui.com/material-ui/qtmui-formcontrollabel/

    API Reference:
    - FormControlLabel API: https://qtmui.com/material-ui/api/form-control-label/
    """

    VALID_LABEL_PLACEMENTS = ["bottom", "end", "start", "top"]

    def __init__(
        self,
        control: Union[State, QWidget],
        checked: Union[State, bool] = False,
        classes: Optional[Union[State, Dict]] = None,
        componentsProps: Optional[Union[State, Dict]] = None,
        disabled: Union[State, bool] = False,
        disableTypography: Union[State, bool] = False,
        inputRef: Optional[Union[State, object]] = None,
        label: Optional[Union[State, str, QWidget, List[QWidget], Callable]] = None,
        labelPlacement: Union[State, str] = "end",
        key: Optional[Union[State, str]] = None,
        onChange: Optional[Union[State, Callable]] = None,
        required: Union[State, bool] = False,
        slotProps: Optional[Union[State, Dict]] = None,
        slots: Optional[Union[State, Dict]] = None,
        sx: Optional[Union[State, List, Dict, Callable, str]] = None,
        value: Optional[Union[State, any]] = None,
        highlight: Union[State, bool] = False,
        fullWidth: Union[State, bool] = False,
        spacing: Union[State, int] = 6,
        **kwargs
    ):
        super().__init__()
        self.setObjectName(f"MuiFormControlLabel-{str(uuid.uuid4())}")

        # Initialize theme
        self.theme = useTheme()

        # Store widget references to prevent Qt deletion
        self._widget_references = []

        # Set properties with validation
        self._set_control(control)
        self._set_checked(checked)
        self._set_classes(classes)
        self._set_componentsProps(componentsProps)
        self._set_disabled(disabled)
        self._set_disableTypography(disableTypography)
        self._set_inputRef(inputRef)
        self._set_label(label)
        self._set_labelPlacement(labelPlacement)
        self._set_key(key)
        self._set_onChange(onChange)
        self._set_required(required)
        self._set_slotProps(slotProps)
        self._set_slots(slots)
        self._set_sx(sx)
        self._set_value(value)
        self._set_highlight(highlight)
        self._set_fullWidth(fullWidth)
        self._set_spacing(spacing)

        # Setup UI
        self._init_ui()

    # Setter and Getter methods
    @_validate_param(file_path="qtmui.material.formcontrollabel", param_name="control", supported_signatures=Union[State, QWidget])
    def _set_control(self, value):
        """Assign value to control."""
        if not isinstance(value, (State, QWidget)):
            raise TypeError(f"control must be a State or QWidget, got {type(value)}")
        self._control = value
        # control = self._get_control()
        # if isinstance(control, QWidget):
        #     if hasattr(control, "setChecked"):
        #         control.setChecked(self._get_checked())
        #     if hasattr(control, "setEnabled"):
        #         control.setEnabled(not self._get_disabled())
        #     if hasattr(control, "setProperty"):
        #         control.setProperty("required", self._get_required())
        #         control.style().unpolish(control)
        #         control.style().polish(control)
        #     if self._get_onChange() and hasattr(control, "stateChanged"):
        #         control.stateChanged.connect(lambda: self.handle_change({"target": {"checked": control.isChecked()}}))
        #     elif self._get_onChange() and hasattr(control, "toggled"):
        #         control.toggled.connect(lambda checked: self.handle_change({"target": {"checked": checked}}))

    def _get_control(self):
        """Get the control value."""
        return self._control.value if isinstance(self._control, State) else self._control

    # @_validate_param(file_path="qtmui.material.formcontrollabel", param_name="checked", supported_signatures=Union[State, bool])
    def _set_checked(self, value):
        """Assign value to checked."""
        self._checked = value

    def _get_checked(self):
        """Get the checked value."""
        return self._checked.value if isinstance(self._checked, State) else self._checked

    @_validate_param(file_path="qtmui.material.formcontrollabel", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
    def _set_classes(self, value):
        """Assign value to classes."""
        self._classes = value

    def _get_classes(self):
        """Get the classes value."""
        return self._classes.value if isinstance(self._classes, State) else self._classes

    @_validate_param(file_path="qtmui.material.formcontrollabel", param_name="componentsProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_componentsProps(self, value):
        """Assign value to componentsProps."""
        self._componentsProps = value

    def _get_componentsProps(self):
        """Get the componentsProps value."""
        return self._componentsProps.value if isinstance(self._componentsProps, State) else self._componentsProps

    @_validate_param(file_path="qtmui.material.formcontrollabel", param_name="disabled", supported_signatures=Union[State, bool])
    def _set_disabled(self, value):
        """Assign value to disabled."""
        self._disabled = value
        control = self._get_control()
        if isinstance(control, QWidget) and hasattr(control, "setEnabled"):
            control.setEnabled(not self._get_disabled())

    def _get_disabled(self):
        """Get the disabled value."""
        return self._disabled.value if isinstance(self._disabled, State) else self._disabled

    @_validate_param(file_path="qtmui.material.formcontrollabel", param_name="disableTypography", supported_signatures=Union[State, bool])
    def _set_disableTypography(self, value):
        """Assign value to disableTypography."""
        self._disableTypography = value

    def _get_disableTypography(self):
        """Get the disableTypography value."""
        return self._disableTypography.value if isinstance(self._disableTypography, State) else self._disableTypography

    @_validate_param(file_path="qtmui.material.formcontrollabel", param_name="inputRef", supported_signatures=Union[State, object, type(None)])
    def _set_inputRef(self, value):
        """Assign value to inputRef."""
        self._inputRef = value

    def _get_inputRef(self):
        """Get the inputRef value."""
        return self._inputRef.value if isinstance(self._inputRef, State) else self._inputRef

    # @_validate_param(file_path="qtmui.material.formcontrollabel", param_name="label", supported_signatures=Union[State, str, QWidget, List[QWidget], type(None)])
    def _set_label(self, value):
        """Assign value to label."""
        self._label = value

    def _get_label(self):
        """Get the label value."""
        return self._label.value if isinstance(self._label, State) else self._label

    @_validate_param(file_path="qtmui.material.formcontrollabel", param_name="labelPlacement", supported_signatures=Union[State, str])
    def _set_labelPlacement(self, value):
        """Assign value to labelPlacement."""
        if value not in self.VALID_LABEL_PLACEMENTS:
            raise ValueError(f"labelPlacement must be one of {self.VALID_LABEL_PLACEMENTS}, got {value}")
        self._labelPlacement = value

    def _get_labelPlacement(self):
        """Get the labelPlacement value."""
        return self._labelPlacement.value if isinstance(self._labelPlacement, State) else self._labelPlacement

    @_validate_param(file_path="qtmui.material.formcontrollabel", param_name="key", supported_signatures=Union[State, str, type(None)])
    def _set_key(self, value):
        """Assign value to key (custom feature)."""
        self._key = value

    def _get_key(self):
        """Get the key value."""
        return self._key.value if isinstance(self._key, State) else self._key

    @_validate_param(file_path="qtmui.material.formcontrollabel", param_name="onChange", supported_signatures=Union[State, Callable, type(None)])
    def _set_onChange(self, value):
        """Assign value to onChange."""
        self._onChange = value

    def _get_onChange(self):
        """Get the onChange value."""
        return self._onChange.value if isinstance(self._onChange, State) else self._onChange

    @_validate_param(file_path="qtmui.material.formcontrollabel", param_name="required", supported_signatures=Union[State, bool])
    def _set_required(self, value):
        """Assign value to required."""
        self._required = value
        # control = self._get_control()
        # if isinstance(control, QWidget) and hasattr(control, "setProperty"):
        #     control.setProperty("required", self._get_required())
        #     control.style().unpolish(control)
        #     control.style().polish(control)

    def _get_required(self):
        """Get the required value."""
        return self._required.value if isinstance(self._required, State) else self._required

    @_validate_param(file_path="qtmui.material.formcontrollabel", param_name="slotProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_slotProps(self, value):
        """Assign value to slotProps."""
        self._slotProps = value

    def _get_slotProps(self):
        """Get the slotProps value."""
        return self._slotProps.value if isinstance(self._slotProps, State) else self._slotProps

    @_validate_param(file_path="qtmui.material.formcontrollabel", param_name="slots", supported_signatures=Union[State, Dict, type(None)])
    def _set_slots(self, value):
        """Assign value to slots."""
        self._slots = value

    def _get_slots(self):
        """Get the slots value."""
        return self._slots.value if isinstance(self._slots, State) else self._slots

    @_validate_param(file_path="qtmui.material.formcontrollabel", param_name="sx", supported_signatures=Union[State, List, Dict, Callable, str, type(None)])
    def _set_sx(self, value):
        """Assign value to sx."""
        self._sx = value

    def _get_sx(self):
        """Get the sx value."""
        return self._sx.value if isinstance(self._sx, State) else self._sx

    # @_validate_param(file_path="qtmui.material.formcontrollabel", param_name="value", supported_signatures=Union[State, any, type(None)])
    def _set_value(self, value):
        """Assign value to value."""
        self._value = value

    def _get_value(self):
        """Get the value value."""
        return self._value.value if isinstance(self._value, State) else self._value

    @_validate_param(file_path="qtmui.material.formcontrollabel", param_name="highlight", supported_signatures=Union[State, bool])
    def _set_highlight(self, value):
        """Assign value to highlight (custom feature)."""
        self._highlight = value

    def _get_highlight(self):
        """Get the highlight value."""
        return self._highlight.value if isinstance(self._highlight, State) else self._highlight

    @_validate_param(file_path="qtmui.material.formcontrollabel", param_name="fullWidth", supported_signatures=Union[State, bool])
    def _set_fullWidth(self, value):
        """Assign value to fullWidth (custom feature)."""
        self._fullWidth = value

    def _get_fullWidth(self):
        """Get the fullWidth value."""
        return self._fullWidth.value if isinstance(self._fullWidth, State) else self._fullWidth

    @_validate_param(file_path="qtmui.material.formcontrollabel", param_name="spacing", supported_signatures=Union[State, int])
    def _set_spacing(self, value):
        """Assign value to spacing (custom feature)."""
        self._spacing = value

    def _get_spacing(self):
        """Get the spacing value."""
        return self._spacing.value if isinstance(self._spacing, State) else self._spacing


    def _init_ui(self):
        self.setObjectName(str(uuid.uuid4()))
        self.setSizePolicy(QSizePolicy.Policy.Maximum, QSizePolicy.Policy.Maximum)

        # if self._fullWidth:
        #     self.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Maximum)
        # else:
        #     self.setSizePolicy(QSizePolicy.Policy.Maximum, QSizePolicy.Policy.Maximum)

        if self._labelPlacement == "start":
            self.setLayout(QHBoxLayout())
            # if self._label is not None:
            #     self.layout().addWidget(QLabel(self._label))
            # if self._control is not None:
            #     self.layout().addWidget(self._control)
        elif self._labelPlacement == "end":
            self.setLayout(QHBoxLayout())
            # self.layout().setAlignment(Qt.AlignmentFlag.AlignLeft)
            # if self._control is not None:
            #     self.layout().addWidget(self._control)
            # if self._label is not None:
            #     self.layout().addWidget(QLabel(self._label))
        elif self._labelPlacement == "top":
            self.setLayout(QVBoxLayout())
            # if self._label is not None:
            #     lb = QLabel(self._label)
            #     lb.setAlignment(Qt.AlignmentFlag.AlignCenter)
            #     self.layout().addWidget(lb)
            # if self._control is not None:
            #     self.layout().addWidget(self._control)
        elif self._labelPlacement == "bottom":
            self.setLayout(QVBoxLayout())
            # if self._control is not None:
            #     self.layout().addWidget(self._control)
            # if self._label is not None:
            #     lb = QLabel(self._label)
            #     lb.setAlignment(Qt.AlignmentFlag.AlignCenter)
            #     self.layout().addWidget(lb)

        # if isinstance(self._label, list) and all(isinstance(item, str) for item in self._label):

        if isinstance(self._label, list):
            frm_label = QFrame()
            frm_label.setLayout(QVBoxLayout())
            label_layout = frm_label.layout()
            self.add_children(layout=label_layout, children=self._label)
            self.layout().addWidget(frm_label)
        elif isinstance(self._label, str):
            self.add_children(self.layout(), Typography(text=self._label))
        elif isinstance(self._label, QWidget):
            self.add_children(self.layout(), self._label)

        if self._fullWidth: # self._sx and isinstance(self._sx, dict) and self._sx.get("width").find("%") != -1
            if self._control is not None:
                if self._labelPlacement == "start" or self._labelPlacement == "top":
                    self.layout().addWidget(HSpacer())
                    self.layout().addWidget(self._control)
                else:
                    self.layout().insertWidget(0, HSpacer())
                    self.layout().insertWidget(0, self._control)
        else:
            if self._control is not None:
                if self._labelPlacement == "start" or self._labelPlacement == "top":
                    self.layout().addWidget(self._control)
                else:
                    self.layout().insertWidget(0, self._control)
        
        if not self._control._key:
            self._control._key = self._key
        if not self._control._value:
            self._control._value = self._value

        # layout = self.layout()
        # if self._label is not None:
        #     self.add_children(layout=layout, children=self._label)

        self.layout().setContentsMargins(0,0,0,0)
        self.layout().setSpacing(self._spacing)

        for widget in self.findChildren(QWidget):
            widget.installEventFilter(self)  # Bắt sự kiện từ con

        if self._tooltip:
            PyWidgetBase._installTooltipFilter(self)

        self.slot_set_stylesheet()
        self.theme = useTheme()
        self.theme.state.valueChanged.connect(self.slot_set_stylesheet)
        self.destroyed.connect(self._on_destroyed)

    def slot_set_stylesheet(self, value=None):
        self._set_stylesheet()

    def add_children(self, layout, children):
        if isinstance(children, str):
            layout.addWidget(QLabel(children), 0, 
                             Qt.AlignmentFlag.AlignLeft if self._labelPlacement == "start" else 
                             Qt.AlignmentFlag.AlignRight if self._labelPlacement == "end" else  
                             Qt.AlignmentFlag.AlignCenter)
        elif isinstance(children, QWidget):
            layout.addWidget(children, 0, Qt.AlignmentFlag.AlignLeft if self._labelPlacement == "start" 
                             else Qt.AlignmentFlag.AlignRight if self._labelPlacement == "end" 
                             else  Qt.AlignmentFlag.AlignCenter)
        elif isinstance(children, list):
            for widget in children:
                if isinstance(widget, QWidget):
                    layout.addWidget(widget, 0, Qt.AlignmentFlag.AlignLeft if self._labelPlacement == "start" 
                                     else Qt.AlignmentFlag.AlignRight if self._labelPlacement == "end" 
                                     else  Qt.AlignmentFlag.AlignCenter)
                    


    # def eventFilter(self, obj, event):
    #     if event.type() == QEvent.MouseButtonPress:
    #         print('prrrrrrrrrr', obj)
    #         return False

    def handle_change(self, event: dict):
        """
        Xử lý sự kiện thay đổi trạng thái của component.

        Args:
            event (dict): Sự kiện thay đổi, lấy trạng thái mới từ event.target.checked.
        """
        if self._onChange:
            self._onChange(event)

    def render(self):
        """
        Hàm render component.
        """
        # Sử dụng control và các thuộc tính đã

    def _set_stylesheet(self):
        theme = useTheme()
        
        sx_qss = ""
        if self._sx:
            if isinstance(self._sx, dict):
                sx_qss = get_qss_style(self._sx, class_name=f"#{self.objectName()}")
            elif isinstance(self._sx, Callable):
                sx = self._sx()
                if isinstance(sx, dict):
                    sx_qss = get_qss_style(sx, class_name=f"#{self.objectName()}")
                elif isinstance(sx, str):
                    sx_qss = sx
            elif isinstance(self._sx, str) and self._sx != "":
                sx_qss = self._sx
                
        stylesheet = f"""
            #{self.objectName()} {{
                color: {theme.palette.text.secondary};
            }}

            {sx_qss}

        """
        self.setStyleSheet(stylesheet)



# src/qtmui/material/image_list_item_bar.py
from PySide6.QtWidgets import QFrame, QLabel, QHBoxLayout, QVBoxLayout
from PySide6.QtCore import Qt

class ImageListItemBar(QFrame):
    def __init__(
        self,
        title: str = "",
        subtitle: str = None,
        actionIcon=None,           # QWidget (ví dụ IconButton)
        actionPosition: str = "right",  # "left" | "right"
        position: str = "bottom",       # "top" | "bottom" | "below"
        sx: dict = None,
        **kwargs
    ):
        super().__init__()
        self.position = position.lower()
        self.setAttribute(Qt.WA_StyledBackground, True)

        # Layout chính
        layout = QHBoxLayout(self)
        layout.setContentsMargins(8, 4, 8, 4)
        layout.setSpacing(8)

        # Action icon (nếu có)
        if actionIcon:
            if actionPosition == "left":
                layout.addWidget(actionIcon)
                layout.addStretch()
            else:
                layout.addStretch()
                layout.addWidget(actionIcon)

        # Text container
        text_layout = QVBoxLayout()
        text_layout.setSpacing(0)

        self.title_label = QLabel(title)
        self.title_label.setStyleSheet("color: white; font-weight: 600; font-size: 14px;")
        text_layout.addWidget(self.title_label)

        if subtitle:
            self.subtitle_label = QLabel(subtitle)
            self.subtitle_label.setStyleSheet("color: rgba(255,255,255,0.7); font-size: 12px;")
            text_layout.addWidget(self.subtitle_label)

        layout.insertLayout(1 if actionPosition == "left" else 0, text_layout)

        # Áp dụng style theo position
        if position in ["top", "bottom"]:
            self.setStyleSheet(f"""
                ImageListItemBar {{
                    background: linear-gradient(to top, 
                        rgba(0,0,0,0.7) 0%, 
                        rgba(0,0,0,0.4) 50%, 
                        transparent 100%);
                    border-radius: {'0 0 8px 8px' if position == 'bottom' else '8px 8px 0 0'};
                }}
            """)
        elif position == "below":
            self.setStyleSheet("""
                ImageListItemBar {
                    background: transparent;
                    padding: 8px 0;
                }
                QLabel { color: rgba(0,0,0,0.87); }
            """)


from PySide6.QtWidgets import QFrame, QHBoxLayout, QTableWidgetItem


class TableWidgetItem(QTableWidgetItem):
    def __init__(
                self,
                selected: bool = False,
                text: str = ""
                ):
        super().__init__()
        self.setText(text)



from PySide6.QtCore import Signal, Qt
from PySide6.QtWidgets import QHeaderView, QLineEdit


class HeaderFilter(QHeaderView):
    filterActivated = Signal()

    def __init__(self, parent):
        super().__init__(Qt.Orientation.Horizontal, parent)
        self._editors = []
        self._padding = 4
        self.setStretchLastSection(True)
        self.setSectionResizeMode(QHeaderView.ResizeMode.Stretch)
        self.setDefaultAlignment(Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter)
        self.setSortIndicatorShown(False)
        self.setSectionsMovable(True)
        self.sectionResized.connect(self.adjustPositions)
        parent.horizontalScrollBar().valueChanged.connect(self.adjustPositions)

    def setFilterBoxes(self, count):
        while self._editors:
            editor = self._editors.pop()
            editor.deleteLater()
        for index in range(count):
            editor = QLineEdit(self.parent())
            editor.setPlaceholderText('Filter')
            editor.setClearButtonEnabled(True)
            editor.returnPressed.connect(self.filterActivated.emit)
            self._editors.append(editor)
        self.adjustPositions()

    def sizeHint(self):
        size = super().sizeHint()
        if self._editors:
            height = self._editors[0].sizeHint().height()
            size.setHeight(size.height() + height + self._padding)
        return size

    def updateGeometries(self):
        if self._editors:
            height = self._editors[0].sizeHint().height()
            self.setViewportMargins(0, 0, 0, height + self._padding)
        else:
            self.setViewportMargins(0, 0, 0, 0)
        super().updateGeometries()
        self.adjustPositions()

    def adjustPositions(self):
        for index, editor in enumerate(self._editors):
            height = editor.sizeHint().height()
            editor.move(
                self.sectionPosition(index) - self.offset() + 2,
                height + (self._padding // 2))
            editor.resize(self.sectionSize(index), height)

    def filterText(self, index):
        if 0 <= index < len(self._editors):
            return self._editors[index].text()
        return ''

    def setFilterText(self, index, text):
        if 0 <= index < len(self._editors):
            self._editors[index].setText(text)

    def clearFilters(self):
        for editor in self._editors:
            editor.clear()



import asyncio
from typing import Callable, Dict, Optional, List, Union
import uuid
from PySide6.QtWidgets import QWidget, QHBoxLayout, QVBoxLayout, QSizePolicy, QFrame
from PySide6.QtCore import Qt, QTimer
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
from qtmui.material.styles import useTheme

# Import lớp Button từ mã của bạn
from .button import Button
from ..widget_base import PyWidgetBase


class ButtonGroup(QFrame, PyWidgetBase):
    def __init__(
        self,
        key: str = None,
        children: List[Button] = None,  # Danh sách các nút Button
        orientation: Qt.Orientation = Qt.Horizontal,  # Hướng của ButtonGroup (ngang hoặc dọc)
        align: Qt.AlignmentFlag = Qt.AlignCenter,  # Căn chỉnh các nút trong nhóm
        spacing: int = 0,  # Khoảng cách giữa các nút (đặt bằng 0 để border không bị ngắt quãng)
        fullWidth: bool = False,  # Nếu là True, các nút sẽ chiếm toàn bộ chiều rộng của container
        parent: Optional[QWidget] = None,
        variant: str = "contained",
        size: str = "medium",
        color: str = "primary",
        sx: Optional[Union[Callable, str, Dict]]= None,
    ):
        super().__init__()

        self.setObjectName(str(uuid.uuid4()))

        self._children = children if children else []
        self._orientation = orientation
        self.align = align
        self.spacing = spacing
        self.fullWidth = fullWidth
        self._color = color
        self._size = size
        self._variant = variant
        self._sx = sx

        # Thiết lập layout dựa trên hướng
        if not self._orientation == "vertical":
            self.setLayout(QHBoxLayout())
        else:
            self.setLayout(QVBoxLayout())

        self.layout().setContentsMargins(0,0,0,0)
        self.layout().setSpacing(self.spacing)
        self.layout().setAlignment(self.align)

        # Thêm các nút vào nhóm và xử lý border
        for i, button in enumerate(self._children):
            try:
                self.add_button(button, i)
            except Exception as e:
                import traceback
                traceback.print_exc()

        self.theme = useTheme()
        self._update_stylesheet()
        self.theme.state.valueChanged.connect(self._update_stylesheet)
        # self.destroyed.connect(lambda ojb: self._on_destroyed())
        self.destroyed.connect(self._on_destroyed)

    def _update_stylesheet(self):
        """Slot to set the stylesheet."""
        self._set_stylesheet()

    def _set_stylesheet(self, component_styled=None):
        theme = useTheme()
        component_styles = theme.components

        borderColor = None
        if self._color != "default":
            borderColor = component_styles["MuiButtonGroup"].get("styles")["root"][self._variant]["first"]["borderColorRender"](self._color)
        else:
            borderColor = component_styles["MuiButtonGroup"].get("styles")["root"][self._variant]["first"]["borderColorDefault"]

        MuiButtonGroup_button_first = component_styles["MuiButtonGroup"].get("styles")["root"][self._variant]["first"]
        MuiButtonGroup_button_first_root_qss = get_qss_style(MuiButtonGroup_button_first)

        MuiButtonGroup_button_first_root_vertical_qss = get_qss_style(MuiButtonGroup_button_first["orientation"]["vertical"])
        MuiButtonGroup_button_first_root_horizontal_qss = get_qss_style(MuiButtonGroup_button_first["orientation"]["horizontal"])

        MuiButtonGroup_button_middle = component_styles["MuiButtonGroup"].get("styles")["root"][self._variant]["middle"]
        MuiButtonGroup_button_middle_root_qss = get_qss_style(MuiButtonGroup_button_middle)
        MuiButtonGroup_button_middle_root_vertical_qss = get_qss_style(MuiButtonGroup_button_middle["orientation"]["vertical"])
        MuiButtonGroup_button_middle_root_horizontal_qss = get_qss_style(MuiButtonGroup_button_middle["orientation"]["horizontal"])

        MuiButtonGroup_button_last = component_styles["MuiButtonGroup"].get("styles")["root"][self._variant]["last"]
        MuiButtonGroup_button_last_root_qss = get_qss_style(MuiButtonGroup_button_last)
        MuiButtonGroup_button_last_root_vertical_qss = get_qss_style(MuiButtonGroup_button_last["orientation"]["vertical"])
        MuiButtonGroup_button_last_root_horizontal_qss = get_qss_style(MuiButtonGroup_button_last["orientation"]["horizontal"])

        borderWidth = ""
        if self._variant == "outlined":
            borderWidth = "border-width: 1px;"

        sx_qss = ""
        if self._sx:
            if isinstance(self._sx, dict):
                sx_qss = get_qss_style(self._sx, class_name=f"#{self.objectName()}")
            elif isinstance(self._sx, Callable):
                sx = self._sx()
                if isinstance(sx, dict):
                    sx_qss = get_qss_style(sx, class_name=f"#{self.objectName()}")
                elif isinstance(sx, str):
                    sx_qss = sx
            elif isinstance(self._sx, str) and self._sx != "":
                sx_qss = self._sx

        # child button styles
        for index, button in enumerate(self._children):

            # Xử lý nút đầu và cuối để tạo border-radius
            if index == 0:
                # Đối với nút đầu tiên
                extra_styles = f"""
                    {
                        f'''
                        #{button.objectName()} {{
                            {MuiButtonGroup_button_first_root_qss}
                            {MuiButtonGroup_button_first_root_vertical_qss if self._orientation == "vertical" else MuiButtonGroup_button_first_root_horizontal_qss}
                            {f"border-color: {borderColor}!important;" if borderColor else ""}
                            {borderWidth}
                            {"border-right-width: 0px;" if self._variant == "outlined" and self._orientation != "vertical" else "border-bottom-width: 0px;" if self._variant == "outlined" and self._orientation == "vertical" else ""}
                        }}
                        '''
                    }
                """

            elif index < len(self._children) - 1:
                # Đối với các nút giữa
                extra_styles = f"""
                    {
                        f'''
                        #{button.objectName()} {{
                            {MuiButtonGroup_button_middle_root_qss}
                            {MuiButtonGroup_button_middle_root_vertical_qss if self._orientation == "vertical" else MuiButtonGroup_button_middle_root_horizontal_qss}
                            {f"border-color: {borderColor};" if borderColor else ""}
                            {borderWidth}
                            {"border-right-width: 0px;" if self._variant == "outlined" and self._orientation != "vertical" else "border-bottom-width: 0px;" if self._variant == "outlined" and self._orientation == "vertical" else ""}
                        }}
                        '''
                    }
                """
            else: 
                # nut cuoi
                extra_styles = f"""
                    {
                        f'''
                        #{button.objectName()} {{
                            {MuiButtonGroup_button_last_root_qss}
                            {MuiButtonGroup_button_last_root_vertical_qss if self._orientation == "vertical" else MuiButtonGroup_button_last_root_horizontal_qss}
                            {f"border-color: {borderColor};" if borderColor else ""}
                            {borderWidth}
                        }}
                        '''
                    }
                """

            button.setStyleSheet(button.styleSheet() + extra_styles + sx_qss)
            
    def add_button(self, button: Button, index: int):
        """Thêm một button vào ButtonGroup với xử lý border cho nút đầu và cuối."""
        if self.fullWidth:
            button.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
        else:
            button.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Preferred)

        button._variant = self._variant
        button._color = self._color
        button._size = self._size

        button._set_stylesheet()
        self.layout().addWidget(button)

    def remove_button(self, button: Button):
        """Loại bỏ một button khỏi ButtonGroup."""
        if button in self._children:
            self._children.remove(button)
            self.layout().removeWidget(button)
            button.setParent(None)

    def set_orientation(self, orientation: Qt.Orientation):
        """Thay đổi hướng của ButtonGroup."""
        self._orientation = orientation
        self.setLayout(QVBoxLayout()) if orientation == Qt.Vertical else self.setLayout(QHBoxLayout())
        self.layout().setSpacing(self.spacing)
        self.layout().setAlignment(self.align)

        # Di chuyển lại các button vào layout mới
        for button in self._children:
            self.layout().addWidget(button)

        self.setLayout(self.layout())

    def set_alignment(self, align: Qt.AlignmentFlag):
        """Thay đổi căn chỉnh của ButtonGroup."""
        self.align = align
        self.layout().setAlignment(align)

    def set_spacing(self, spacing: int):
        """Thay đổi khoảng cách giữa các nút."""
        self.spacing = spacing
        self.layout().setSpacing(spacing)

    def set_fullWidth(self, fullWidth: bool):
        """Thay đổi thuộc tính fullWidth cho các nút trong nhóm."""
        self.fullWidth = fullWidth
        for button in self._children:
            if fullWidth:
                button.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
            else:
                button.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Preferred)




from typing import Any, Callable, Dict, Optional
from .use_state import useState, State
from dataclasses import dataclass
from PySide6.QtCore import QObject, Signal

@dataclass
class UseFormType:
    state: State
    handleSubmit: Callable[..., any]
    setValue: Callable[..., any]
    formState: Callable[..., any]
    control: dict
    watch: Callable[..., dict]
    reset: Callable[..., any]
    resolver: Callable[..., any]
    fields: State

# Context toàn cục để lưu trữ UseForm instance
class FormContext(QObject):
    _instance = None

    @classmethod
    def setInstance(cls, instance: 'UseForm'):
        cls._instance = instance

    @classmethod
    def getInstance(cls) -> Optional['UseForm']:
        return cls._instance

# Định nghĩa lại UseForm để tương thích với context
class UseForm(QObject):
    controlChanged = Signal(object)
    stateChanged = Signal(object)

    def __init__(self, resolver=None, values=None):
        super().__init__()
        if isinstance(values, State):
            self.state: State = values
        elif isinstance(values, dict):
            self.state, _ = useState(values)
        self.data, self.setData = useState(self.state.value)
        self.resolver = resolver
        self.fields, self.setFields = useState([])
        self.isReset, self.setIsReset = useState(False)
        FormContext.setInstance(self)  # Lưu instance vào context

    def handleSubmit(self, *args, **kwargs):
        self.setData(True)

    def setValue(self, name: str, value: Any):
        current_data = self.state.value.copy()
        current_data[name] = value
        self.setData(current_data)

    def watch(self) -> State:
        return self.data

    def reset(self):
        self.setData({})
        self.setIsReset(True)

    def formState(self) -> State:
        return self.state

    def control(self) -> Dict[str, Any]:
        return self.state.value

def useForm(resolver: Callable=None, values: object=None)->UseFormType:
    """
    Hàm này trả về một instance của Popover,
    tương tự như cách bạn sử dụng hook trong React.
    """
    return UseForm(resolver, values)


@dataclass
class UseFormContextType:
    # state: State
    # handleSubmit: Callable[..., any]
    # setValue: Callable[..., any]
    # formState: Callable[..., any]
    # control: dict
    # watch: Callable[..., dict]
    # reset: Callable[..., any]
    # resolver: Callable[..., any]
    control: Optional[Any]
    setValue: Callable[..., any]
    watch: Callable[..., dict]
    resetField: Callable[..., any]

# Hàm useFormContext
def useFormContext() -> UseFormContextType:
    instance = FormContext.getInstance()
    if not instance:
        raise ValueError("useFormContext must be used within a FormProvider")
    # return UseFormType(
    #     state=instance.state,
    #     handleSubmit=instance.handleSubmit,
    #     setValue=instance.setValue,
    #     formState=instance.formState,
    #     control=instance.control,
    #     watch=instance.watch,
    #     reset=instance.reset,
    #     resolver=instance.resolver,
    # )
    context = UseFormContextType(
        control=instance,
        setValue=instance.setValue,
        watch=instance.watch,
        resetField=instance.reset,
    )
    return context





from __future__ import annotations

import warnings
import uuid
from PySide6.QtCore import QByteArray
from PySide6.QtSvgWidgets import QSvgWidget
from PySide6.QtWidgets import QSizePolicy

from qtmui.lib.iconify_client.iconify_client_qt import iconify_client
from qtmui.lib.iconify_client.iconify_cache import svg_cache, cache_key


class PySvgWidget(QSvgWidget):

    def __init__(self, key=None, color=None, flip=None, rotate=None,
                 width=None, height=None, size=None):
        super().__init__()
        self.setObjectName(str(uuid.uuid4()))

        self._key = key
        self._color = color
        self._flip = flip
        self._rotate = rotate
        self._width = width
        self._height = height

        self.client = iconify_client()
        self.client.apiFinished.connect(self._on_iconify)
        self.client.apiError.connect(self._on_iconify_error)

        self.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)

        if key:
            self.loadIcon()

    # ---------------------------------------------------------
    def loadIcon(self):
        if not self._key or ":" not in self._key:
            warnings.warn(f"Invalid Iconify key: {self._key}")
            return

        prefix, name = self._key.split(":", 1)

        args = (prefix, name)
        kwargs = {
            "color": self._color,
            "width": self._width,
            "height": self._height,
            "flip": self._flip,
            "rotate": self._rotate,
        }

        self._cache_id = cache_key(args, kwargs, "0")

        cache = svg_cache()
        if self._cache_id in cache:
            self._set_svg(cache[self._cache_id])
            return

        self.client.svg(prefix, name, **kwargs)

    # ---------------------------------------------------------
    def _on_iconify(self, key: str, data: bytes):
        if key != f"svg:{self._cache_id}":
            return

        cache = svg_cache()
        cache[self._cache_id] = data
        self._set_svg(data)

    # ---------------------------------------------------------
    def _on_iconify_error(self, key: str, msg: str):
        if key != f"svg:{self._cache_id}":
            return
        warnings.warn(f"Iconify error for {self._key}: {msg}")
        self._set_svg(b"<svg/>")

    # ---------------------------------------------------------
    def _set_svg(self, data: bytes):
        self.load(QByteArray(data))
        if self._width and self._height:
            self.setFixedSize(self._width, self._height)



import inspect
from typing import Callable, Optional, List
import hashlib
from functools import lru_cache, partial
from PySide6.QtCore import QObject, Signal
from .use_state import State, useState

class MemoHook(QObject):
    valueChanged = Signal(object)

    def __init__(self):
        super().__init__()
        self.cache = {}

    def _hash_deps(self, deps: List) -> str:
        """
        Tạo hash từ danh sách các dependencies.
        (Ở đây, ta dùng hàm str(deps) để chuyển đổi; nếu dependencies không thể convert sang string an toàn,
         hãy thay đổi hàm này cho phù hợp.)
        """
        return hashlib.md5(str(deps).encode()).hexdigest()

    def use_memo(self, fn: Callable, deps: List) -> any:
        deps_hash = self._hash_deps(deps)
        if deps_hash in self.cache:
            return self.cache[deps_hash]["value"]
        else:
            value = fn()
            self.cache[deps_hash] = {"value": value}
            return value

    def exec(self, fn: Callable, deps: Optional[List[State]]):
        """
        Khởi tạo state dựa trên giá trị tính từ callback (dựa trên dependencies)
        Và kết nối tín hiệu valueChanged của từng dependency để cập nhật state khi có thay đổi.
        """
        if deps:
            # Kiểm tra số lượng tham số của callback
            sig = inspect.signature(fn)
            if len(sig.parameters) == 0:
                # Nếu callback không nhận đối số, gọi fn() dù có deps
                value, setValue = useState(fn())
                for dep in deps:
                    dep.valueChanged.connect(lambda: setValue(fn()))
            else:
                # Nếu callback nhận đối số, truyền danh sách dependencies
                value, setValue = useState(fn(*deps))
                for dep in deps:
                    # dep.valueChanged.connect(lambda: setValue(fn(*deps)))
                    dep.valueChanged.connect(lambda *args, **kwargs: setValue(fn(*args, **kwargs)))
        else:
            value, setValue = useState(fn())
        return value

def useMemo(callback: Callable = None, dependencies: Optional[List[State]] = None) -> State:
    """
    Hook useMemo nhận callback và danh sách dependencies.
    Giá trị trả về sẽ được tính toán lại chỉ khi một trong các dependency thay đổi.
    """
    memoHook = MemoHook()
    value = memoHook.exec(callback, dependencies)
    return value



from typing import Callable
from PySide6.QtWidgets import QToolButton, QColorDialog, QDialogButtonBox, QGridLayout, QSpinBox
from PySide6.QtCore import Slot, Qt

from qtmui.material.styles import useTheme
from qtmui.i18n.use_translation import translate, i18n

class ColorPicker(QToolButton):

    def __init__(
            self,
            tooltip: str = None,
            onChange: Callable = None
    ):
        super().__init__()

        self._tooltip = tooltip
        self._onChange = onChange
        self._dialog_color = QColorDialog(self)

        self.initUI()
        i18n.langChanged.connect(self.reTranslation)
        self.reTranslation()
    
    def initUI(self):
        self.setCursor(Qt.PointingHandCursor)
        self.clicked.connect(self.on_click)
        self._set_color()

        theme = useTheme()
        theme.state.valueChanged.connect(self._set_theme)
        self._set_theme()

    def reTranslation(self):
        if self._tooltip:
            if isinstance(self._tooltip, Callable):
                self.setToolTip(translate(self._tooltip))
            else:
                self.setToolTip(self._tooltip)

    def _set_color(self, color=None):
        theme = useTheme()
        self.setStyleSheet(f"margin-left: 8px;border: 0px solid transparent;border-radius: 2px;background-color: {color or theme.palette.grey._500};")

    def _set_theme(self):
        theme = useTheme()

        # self._dialog_color.setOption(QColorDialog.ShowAlphaChannel)

        grid = self._dialog_color.findChild(QGridLayout)
        # print('grid____________________', grid)
        names = iter(('hue', 'sat', 'val', 'red', 'green', 'blue', 'alpha'))
        for i in range(grid.count()):
            item = grid.itemAt(i)
            widget = item.widget()
            if isinstance(widget, QSpinBox):
                widget.setStyleSheet('background: red;')
                widget.setObjectName(next(names))

        # alternatively:
        #spins = dialog.findChildren(QtWidgets.QSpinBox)
        #names = 'hue', 'sat', 'val', 'red', 'green', 'blue', 'alpha'
        #for name, spin in zip(names, spins):
        #    spin.setObjectName(name)

        buttonBox = self._dialog_color.findChild(QDialogButtonBox)
        buttonBox.button(QDialogButtonBox.Ok).setObjectName('ok')
        buttonBox.button(QDialogButtonBox.Cancel).setObjectName('cancel')

        self._dialog_color.setStyleSheet(f'''
            /* the HTML color line edit */
            QColorDialog {{
                background: yellow;
            }}
            QLineEdit {{
                background: yellow;
            }}

            /* the spin boxes */
            QSpinBox#hue {{
                background: coral;
            }}
            QSpinBox#sat {{
                background: orange;
            }}
            QSpinBox#val {{
                background: lightgray;
            }}
            QSpinBox#red {{
                background: orangered;
            }}
            QSpinBox#green {{
                background: lime;
            }}
            QSpinBox#blue {{
                background: aqua;
            }}
            QSpinBox#alpha {{
                background: pink;
            }}

            /* buttons that are children of QDialogButtonBox */
            QDialogButtonBox QAbstractButton#ok {{
                background: green;
            }}
            QDialogButtonBox QAbstractButton#cancel {{
                background: red;
            }}
            ''')

        # self._dialog_color.setStyleSheet(
        #     f"""
        #         QColorPicker {{
        #             background-color: rgb(6, 6, 14);
        #         }}
        #         QPushButton {{
        #             color: rgb(211, 213, 201); 
        #             background-color: rgb(36, 36, 44);
        #         }}
        #         QLabel {{
        #             color: rgb(211, 213, 201); 
        #             background-color: rgb(6, 6, 14);
        #         }}
        #         QLineEdit {{
        #             color: rgb(211, 213, 201); 
        #             background-color: rgb(36, 36, 44);
        #         }}
        #         QSpinBox {{
        #             color: rgb(211, 213, 201); 
        #             background-color: rgb(36, 36, 44);
        #         }}
        #     """
        # )

    @Slot()
    def on_click(self):
        self.openColorDialog()
        self._set_theme()

    def openColorDialog(self):
        color = self._dialog_color.getColor()

        if color.isValid():
            print(color.name())
            self._onChange(color.name())
            self._set_color(color.name())





import uuid
from typing import Optional, Union, Dict, List, Callable
from PySide6.QtWidgets import QHBoxLayout, QLabel, QSizePolicy, QFrame
from PySide6.QtCore import Qt, Signal
from qtmui.hooks import State
from ..button.icon_button import IconButton
from ..button.button import Button
from qtmui.material.styles import useTheme
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
from ..widget_base import PyWidgetBase
from ..py_iconify import PyIconify, Iconify
from ...qtmui_assets import QTMUI_ASSETS
from ..utils.validate_params import _validate_param

class Pagination(QFrame, PyWidgetBase):
    """
    A component that renders a pagination control, styled like Material-UI Pagination.

    The `Pagination` component allows users to navigate through pages, with support for boundary and sibling pages,
    custom rendering, and accessibility features. It integrates with `Button` and `IconButton` components in the `qtmui`
    framework, retaining all existing parameters and aligning with MUI props.

    Parameters
    ----------
    boundaryCount : State or int, optional
        Number of always visible pages at the beginning and end. Default is 1.
        Can be a `State` object for dynamic updates.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    color : State or str, optional
        The active color ("primary", "secondary", "standard", or custom). Default is "standard".
        Can be a `State` object for dynamic updates.
    count : State or int, optional
        The total number of pages. Default is 1.
        Can be a `State` object for dynamic updates.
    defaultPage : State or int, optional
        The page selected by default when uncontrolled. Default is 1.
        Can be a `State` object for dynamic updates.
    disabled : State or bool, optional
        If True, the component is disabled. Default is False.
        Can be a `State` object for dynamic updates.
    getItemAriaLabel : State or Callable, optional
        Function returning a user-friendly name for the current page. Default is None.
        Can be a `State` object for dynamic updates.
        Signature: (type: str, page: int | None, selected: bool) -> str
    hideNextButton : State or bool, optional
        If True, hide the next-page button. Default is False.
        Can be a `State` object for dynamic updates.
    hidePrevButton : State or bool, optional
        If True, hide the previous-page button. Default is False.
        Can be a `State` object for dynamic updates.
    onChange : State or Callable, optional
        Callback fired when the page changes. Default is None.
        Can be a `State` object for dynamic updates.
        Signature: (event: Any, page: int) -> None
    page : State or int, optional
        The current page (starts from 1). Default is None (uses defaultPage).
        Can be a `State` object for dynamic updates.
    renderItem : State or Callable, optional
        Function to render pagination items. Default is None (uses Button/IconButton).
        Can be a `State` object for dynamic updates.
        Signature: (params: Dict) -> QWidget
    shape : State or str, optional
        Shape of pagination items ("circular" or "rounded"). Default is "circular".
        Can be a `State` object for dynamic updates.
    showFirstButton : State or bool, optional
        If True, show the first-page button. Default is False.
        Can be a `State` object for dynamic updates.
    showLastButton : State or bool, optional
        If True, show the last-page button. Default is False.
        Can be a `State` object for dynamic updates.
    siblingCount : State or int, optional
        Number of always visible pages before and after the current page. Default is 1.
        Can be a `State` object for dynamic updates.
    size : State or str, optional
        Size of the component ("small", "medium", "large"). Default is "medium".
        Can be a `State` object for dynamic updates.
    sx : State, list, dict, Callable, str, or None, optional
        System prop for CSS overrides and additional styles. Default is None.
        Can be a `State` object for dynamic updates.
    variant : State or str, optional
        Variant of the component ("outlined" or "text"). Default is "text".
        Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to the parent `QFrame` class,
        supporting props of the native component (e.g., id, className).

    Signals
    -------
    pageChanged : Signal
        Emitted when the page changes, carrying the new page number.

    Notes
    -----
    - All existing parameters from the previous implementation are retained, with `enabled` replaced by `disabled`.
    - Props of the native component are supported via `**kwargs`.
    - The `page` prop starts numbering from 1, consistent with MUI Pagination.
    - MUI classes applied: `MuiPagination-root`, `MuiPagination-outlined`, `MuiPagination-text`.
    - Integrates with `Button`, `IconButton`, and `PyIconify` for consistent styling.

    Demos:
    - Pagination: https://qtmui.com/material-ui/qtmui-pagination/

    API Reference:
    - Pagination API: https://qtmui.com/material-ui/api/pagination/
    """

    pageChanged = Signal(int)

    VALID_COLORS = ["primary", "secondary", "standard"]
    VALID_SHAPES = ["circular", "rounded"]
    VALID_SIZES = ["small", "medium", "large"]
    VALID_VARIANTS = ["outlined", "text"]

    def __init__(
        self,
        boundaryCount: Union[State, int] = 1,
        classes: Optional[Union[State, Dict]] = None,
        color: Union[State, str] = "default",
        count: Union[State, int] = 1,
        defaultPage: Union[State, int] = 1,
        disabled: Union[State, bool] = False,
        getItemAriaLabel: Optional[Union[State, Callable]] = None,
        hideNextButton: Union[State, bool] = False,
        hidePrevButton: Union[State, bool] = False,
        onChange: Optional[Union[State, Callable]] = None,
        page: Optional[Union[State, int]] = None,
        renderItem: Optional[Union[State, Callable]] = None,
        shape: Union[State, str] = "circular",
        showFirstButton: Union[State, bool] = False,
        showLastButton: Union[State, bool] = False,
        siblingCount: Union[State, int] = 1,
        size: Union[State, str] = "medium",
        sx: Optional[Union[State, List, Dict, Callable, str]] = None,
        variant: Union[State, str] = "text",
        *args,
        **kwargs
    ):
        super().__init__(*args, **kwargs)
        self.setObjectName(f"Pagination-{str(uuid.uuid4())}")

        self.theme = useTheme()
        self._button_references = []

        # Set properties with validation
        self._set_boundaryCount(boundaryCount)
        self._set_classes(classes)
        self._set_color(color)
        self._set_count(count)
        self._set_defaultPage(defaultPage)
        self._set_disabled(disabled)
        self._set_getItemAriaLabel(getItemAriaLabel)
        self._set_hideNextButton(hideNextButton)
        self._set_hidePrevButton(hidePrevButton)
        self._set_onChange(onChange)
        self._set_page(page)
        self._set_renderItem(renderItem)
        self._set_shape(shape)
        self._set_showFirstButton(showFirstButton)
        self._set_showLastButton(showLastButton)
        self._set_siblingCount(siblingCount)
        self._set_size(size)
        self._set_sx(sx)
        self._set_variant(variant)

        self._current_page = self._get_page() or self._get_defaultPage()

        self._init_ui()


    # Setter and Getter methods
    @_validate_param(
        file_path="qtmui.material.pagination",
        param_name="boundaryCount",
        supported_signatures=Union[State, int],
        validator=lambda x: x > 0 if isinstance(x, int) else True
    )
    def _set_boundaryCount(self, value):
        """Assign value to boundaryCount."""
        self._boundaryCount = value

    def _get_boundaryCount(self):
        """Get the boundaryCount value."""
        return self._boundaryCount.value if isinstance(self._boundaryCount, State) else self._boundaryCount

    @_validate_param(
        file_path="qtmui.material.pagination",
        param_name="classes",
        supported_signatures=Union[State, Dict, type(None)]
    )
    def _set_classes(self, value):
        """Assign value to classes."""
        self._classes = value

    def _get_classes(self):
        """Get the classes value."""
        return self._classes.value if isinstance(self._classes, State) else self._classes

    @_validate_param(
        file_path="qtmui.material.pagination",
        param_name="color",
        supported_signatures=Union[State, str]
    )
    def _set_color(self, value):
        """Assign value to color."""
        self._color = value

    def _get_color(self):
        """Get the color value."""
        return self._color.value if isinstance(self._color, State) else self._color

    @_validate_param(
        file_path="qtmui.material.pagination",
        param_name="count",
        supported_signatures=Union[State, int],
        validator=lambda x: x > 0 if isinstance(x, int) else True
    )
    def _set_count(self, value):
        """Assign value to count."""
        self._count = value

    def _get_count(self):
        """Get the count value."""
        return self._count.value if isinstance(self._count, State) else self._count

    @_validate_param(
        file_path="qtmui.material.pagination",
        param_name="defaultPage",
        supported_signatures=Union[State, int],
        validator=lambda x: x > 0 if isinstance(x, int) else True
    )
    def _set_defaultPage(self, value):
        """Assign value to defaultPage."""
        self._defaultPage = value

    def _get_defaultPage(self):
        """Get the defaultPage value."""
        return self._defaultPage.value if isinstance(self._defaultPage, State) else self._defaultPage

    @_validate_param(
        file_path="qtmui.material.pagination",
        param_name="disabled",
        supported_signatures=Union[State, bool]
    )
    def _set_disabled(self, value):
        """Assign value to disabled."""
        self._disabled = value

    def _get_disabled(self):
        """Get the disabled value."""
        return self._disabled.value if isinstance(self._disabled, State) else self._disabled

    @_validate_param(
        file_path="qtmui.material.pagination",
        param_name="getItemAriaLabel",
        supported_signatures=Union[State, Callable, type(None)]
    )
    def _set_getItemAriaLabel(self, value):
        """Assign value to getItemAriaLabel."""
        self._getItemAriaLabel = value

    def _get_getItemAriaLabel(self):
        """Get the getItemAriaLabel value."""
        return self._getItemAriaLabel.value if isinstance(self._getItemAriaLabel, State) else self._getItemAriaLabel

    @_validate_param(
        file_path="qtmui.material.pagination",
        param_name="hideNextButton",
        supported_signatures=Union[State, bool]
    )
    def _set_hideNextButton(self, value):
        """Assign value to hideNextButton."""
        self._hideNextButton = value

    def _get_hideNextButton(self):
        """Get the hideNextButton value."""
        return self._hideNextButton.value if isinstance(self._hideNextButton, State) else self._hideNextButton

    @_validate_param(
        file_path="qtmui.material.pagination",
        param_name="hidePrevButton",
        supported_signatures=Union[State, bool]
    )
    def _set_hidePrevButton(self, value):
        """Assign value to hidePrevButton."""
        self._hidePrevButton = value

    def _get_hidePrevButton(self):
        """Get the hidePrevButton value."""
        return self._hidePrevButton.value if isinstance(self._hidePrevButton, State) else self._hidePrevButton

    @_validate_param(
        file_path="qtmui.material.pagination",
        param_name="onChange",
        supported_signatures=Union[State, Callable, type(None)]
    )
    def _set_onChange(self, value):
        """Assign value to onChange."""
        self._onChange = value

    def _get_onChange(self):
        """Get the onChange value."""
        return self._onChange.value if isinstance(self._onChange, State) else self._onChange

    @_validate_param(
        file_path="qtmui.material.pagination",
        param_name="page",
        supported_signatures=Union[State, int, type(None)],
        validator=lambda x: x > 0 if isinstance(x, int) else True
    )
    def _set_page(self, value):
        """Assign value to page."""
        self._page = value

    def _get_page(self):
        """Get the page value."""
        return self._page.value if isinstance(self._page, State) else self._page

    @_validate_param(
        file_path="qtmui.material.pagination",
        param_name="renderItem",
        supported_signatures=Union[State, Callable, type(None)]
    )
    def _set_renderItem(self, value):
        """Assign value to renderItem."""
        self._renderItem = value

    def _get_renderItem(self):
        """Get the renderItem value."""
        return self._renderItem.value if isinstance(self._renderItem, State) else self._renderItem

    @_validate_param(
        file_path="qtmui.material.pagination",
        param_name="shape",
        supported_signatures=Union[State, str],
        valid_values=VALID_SHAPES
    )
    def _set_shape(self, value):
        """Assign value to shape."""
        self._shape = value

    def _get_shape(self):
        """Get the shape value."""
        return self._shape.value if isinstance(self._shape, State) else self._shape

    @_validate_param(
        file_path="qtmui.material.pagination",
        param_name="showFirstButton",
        supported_signatures=Union[State, bool]
    )
    def _set_showFirstButton(self, value):
        """Assign value to showFirstButton."""
        self._showFirstButton = value

    def _get_showFirstButton(self):
        """Get the showFirstButton value."""
        return self._showFirstButton.value if isinstance(self._showFirstButton, State) else self._showFirstButton

    @_validate_param(
        file_path="qtmui.material.pagination",
        param_name="showLastButton",
        supported_signatures=Union[State, bool]
    )
    def _set_showLastButton(self, value):
        """Assign value to showLastButton."""
        self._showLastButton = value

    def _get_showLastButton(self):
        """Get the showLastButton value."""
        return self._showLastButton.value if isinstance(self._showLastButton, State) else self._showLastButton

    @_validate_param(
        file_path="qtmui.material.pagination",
        param_name="siblingCount",
        supported_signatures=Union[State, int],
        validator=lambda x: x >= 0 if isinstance(x, int) else True
    )
    def _set_siblingCount(self, value):
        """Assign value to siblingCount."""
        self._siblingCount = value

    def _get_siblingCount(self):
        """Get the siblingCount value."""
        return self._siblingCount.value if isinstance(self._siblingCount, State) else self._siblingCount

    @_validate_param(
        file_path="qtmui.material.pagination",
        param_name="size",
        supported_signatures=Union[State, str],
        valid_values=VALID_SIZES
    )
    def _set_size(self, value):
        """Assign value to size."""
        self._size = value

    def _get_size(self):
        """Get the size value."""
        return self._size.value if isinstance(self._size, State) else self._size

    @_validate_param(
        file_path="qtmui.material.pagination",
        param_name="sx",
        supported_signatures=Union[State, List, Dict, Callable, str, type(None)]
    )
    def _set_sx(self, value):
        """Assign value to sx."""
        self._sx = value

    def _get_sx(self):
        """Get the sx value."""
        return self._sx.value if isinstance(self._sx, State) else self._sx

    @_validate_param(
        file_path="qtmui.material.pagination",
        param_name="variant",
        supported_signatures=Union[State, str],
        valid_values=VALID_VARIANTS
    )
    def _set_variant(self, value):
        """Assign value to variant."""
        self._variant = value

    def _get_variant(self):
        """Get the variant value."""
        return self._variant.value if isinstance(self._variant, State) else self._variant


    def _init_ui(self):
        self.setObjectName("PyPagination")

        self.setLayout(QHBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)
        self.setSizePolicy(QSizePolicy.Maximum, QSizePolicy.Maximum)

        # Previous button
        if self._shape == "circular":
            # self.prev_button = IconButton(icon=PyIconify(key=QTMUI_ASSETS.ICONS.ARROW_LEFT), size=self._size, variant="text", color=self._color)
            self.prev_button = IconButton(icon=Iconify(key="ri:arrow-left-s-line"), size=self._size, variant="text", color=self._color)
        else:
            self.prev_button = Button(startIcon=Iconify(key="ri:arrow-left-s-line"), size=self._size, variant="text", color=self._color)

        self.prev_button.clicked.connect(self.prev_page)
        self.layout().addWidget(self.prev_button)

        # Next button
        if self._shape == "circular":
            # self.next_button = IconButton(icon=PyIconify(key=QTMUI_ASSETS.ICONS.ARROW_RIGHT), size=self._size, variant="text", color=self._color)
            self.next_button = IconButton(icon=Iconify(key="ri:arrow-right-s-line"), size=self._size, variant="text", color=self._color)
        else:
            # self.next_button = Button(startIcon=PyIconify(key=QTMUI_ASSETS.ICONS.ARROW_RIGHT), size=self._size, variant="text", color=self._color)
            self.next_button = Button(startIcon=Iconify(key="ri:arrow-right-s-line"), size=self._size, variant="text", color=self._color)

        self.next_button.clicked.connect(self.next_page)

        # Page buttons
        self.page_buttons = []
        self.create_page_buttons()

        self.update_buttons()

        # self.setAttribute(Qt.WA_TransparentForMouseEvents)

        self.slot_set_stylesheet()
        self.theme = useTheme()
        self.theme.state.valueChanged.connect(self.slot_set_stylesheet)
        self.destroyed.connect(self._on_destroyed)

    def slot_set_stylesheet(self, value=None):
        self._set_stylesheet()

    def _set_stylesheet(self):
        theme = useTheme()
        component_styles = theme.components

        PyPagination_styles_root_qss = get_qss_style(component_styles["PyPagination"].get("styles")["root"][self._color])
        # print('PyPagination_styles_root_qss___________', PyPagination_styles_root_qss)

        for button in self.page_buttons:
            if isinstance(button, Button):
                if (button.text().isdigit() and int(button.text()) == self._current_page) :
                    button.set_selected(True)
                else:
                    button.set_selected(False)


    def create_page_buttons(self):
        # Clear existing buttons
        for btn in self.page_buttons:
            self.layout().removeWidget(btn)
            btn.deleteLater()
        self.page_buttons.clear()

        # Create page buttons
        if self._count <= 7:
            pages = range(1, self._count + 1)
        else:
            if self._current_page <= 4:
                pages = list(range(1, 6)) + ["..."] + [self._count]
            elif self._current_page > self._count - 4:
                pages = [1] + ["..."] + list(range(self._count - 4, self._count + 1))
            else:
                pages = [1] + ["..."] + list(range(self._current_page - 1, self._current_page + 2)) + ["..."] + [self._count]

        for page in pages:
            if page == "...":
                btn = QLabel(page)
                btn.setAlignment(Qt.AlignCenter)
            else:
                btn = Button(
                    text=str(page), 
                    size=self._size, 
                    variant=self._variant, 
                    color=self._color, 
                    sx=f"""
                        Button {{
                            border-radius: {"15px" if self._size == "small" else "18px" if self._size == "medium" else "24px"};
                        }}
                    """
                )
                btn.setMinimumWidth(30 if self._size == "small" else 36)
                btn.clicked.connect(lambda p=page: self.go_to_page(p))
            self.layout().addWidget(btn)
            self.page_buttons.append(btn)
        self.layout().insertWidget(-1, self.next_button)

        self._set_stylesheet()


    def update_buttons(self):
        self.create_page_buttons()
        # for btn in self.page_buttons:
        #     try:
        #         if isinstance(btn, Button) and (btn.label.text().isdigit() and int(btn.label.text()) == self._current_page) :
        #             # btn.setStyleSheet("background-color: #1c1c1c; color: white; border-radius: 50%;")
        #             print('zoooooooooooo_____select')
        #             btn.set_selected(True)
        #         elif isinstance(btn, Button) :
        #             # btn.setStyleSheet("")
        #             btn.set_selected(False)
        #     except Exception as e:
        #         print('______666____', btn.text())


        self.prev_button.setEnabled(self._current_page > 1)
        self.next_button.setEnabled(self._current_page < self._count)

    def go_to_page(self, page):
        self._current_page = page
        self.update_buttons()

    def prev_page(self):
        if self._current_page > 1:
            self._current_page -= 1
            self.update_buttons()

    def next_page(self):
        if self._current_page < self._count:
            self._current_page += 1
            self.update_buttons()





from typing import Optional, Union, Dict, Callable, List
from PySide6.QtWidgets import QFrame, QHBoxLayout, QWidget
from PySide6.QtCore import Qt
import uuid
from qtmui.hooks import State
from ...material.styles import useTheme
from ..typography import Typography
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
from ..utils.validate_params import _validate_param

class FormLabel(QFrame):
    """
    A component that provides a label for form controls.

    The `FormLabel` component is used to display labels for form controls, supporting all
    props of the Material-UI `FormLabel` component.

    Parameters
    ----------
    children : State, str, QWidget, List[QWidget], or None, optional
        The content of the component, typically text or a widget. Default is None.
        Can be a `State` object for dynamic updates.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    color : State or str, optional
        The color of the component ("error", "info", "primary", "secondary", "success", "warning",
        or custom color). Default is "primary".
        Can be a `State` object for dynamic updates.
    component : State or str, optional
        The component used for the root node (e.g., "label"). Default is None (uses QFrame).
        Can be a `State` object for dynamic updates.
    disabled : State or bool, optional
        If True, the label is displayed in a disabled state. Default is False.
        Can be a `State` object for dynamic updates.
    error : State or bool, optional
        If True, the label is displayed in an error state. Default is False.
        Can be a `State` object for dynamic updates.
    filled : State or bool, optional
        If True, the label uses filled classes key. Default is False.
        Can be a `State` object for dynamic updates.
    focused : State or bool, optional
        If True, the label uses focused classes key. Default is False.
        Can be a `State` object for dynamic updates.
    required : State or bool, optional
        If True, the label indicates that the input is required (adds *). Default is False.
        Can be a `State` object for dynamic updates.
    sx : State, list, dict, Callable, str, or None, optional
        System prop for CSS overrides and additional styles. Default is None.
        Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to the parent `QFrame` class,
        supporting props of the native component (e.g., parent, style, className).

    Attributes
    ----------
    VALID_COLORS : list[str]
        Valid values for `color`: ["error", "info", "primary", "secondary", "success", "warning"].

    Notes
    -----
    - Props of the native component are supported via `**kwargs` (e.g., `parent`, `style`, `className`).
    - If `children` is a string, it is rendered using `Typography` with variant="button".
    - If `children` is a `QWidget` or list of `QWidget`, it is added directly to the layout.

    Demos:
    - FormLabel: https://qtmui.com/material-ui/qtmui-formlabel/

    API Reference:
    - FormLabel API: https://qtmui.com/material-ui/api/form-label/
    """

    VALID_COLORS = ["error", "info", "primary", "secondary", "success", "warning"]

    def __init__(
        self,
        children: Optional[Union[State, str, QWidget, List[QWidget]]] = None,
        classes: Optional[Union[State, Dict]] = None,
        color: Union[State, str] = "primary",
        component: Optional[Union[State, str]] = None,
        disabled: Union[State, bool] = False,
        label: Optional[Union[State, str, Callable]] = None,
        error: Union[State, bool] = False,
        filled: Union[State, bool] = False,
        focused: Union[State, bool] = False,
        required: Union[State, bool] = False,
        sx: Optional[Union[State, List, Dict, Callable, str]] = None,
        **kwargs
    ):
        super().__init__(**kwargs)
        self.setObjectName(f"MuiFormLabel-{str(uuid.uuid4())}")

        # Initialize theme
        self.theme = useTheme()

        # Store widget references to prevent Qt deletion
        self._widget_references = []

        # Set properties with validation
        self._set_children(children)
        self._set_classes(classes)
        self._set_color(color)
        self._set_component(component)
        self._set_disabled(disabled)
        self._set_error(error)
        self._set_label(label)
        self._set_filled(filled)
        self._set_focused(focused)
        self._set_required(required)
        self._set_sx(sx)

        # Setup UI
        self._init_ui()


    # Setter and Getter methods
    # @_validate_param(file_path="qtmui.material.formlabel", param_name="children", supported_signatures=Union[State, str, QWidget, List[QWidget], type(None)])
    def _set_children(self, value):
        """Assign value to children."""
        self._children = value

    def _get_children(self):
        """Get the children value."""
        return self._children.value if isinstance(self._children, State) else self._children

    @_validate_param(file_path="qtmui.material.formlabel", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
    def _set_classes(self, value):
        """Assign value to classes."""
        self._classes = value

    def _get_classes(self):
        """Get the classes value."""
        return self._classes.value if isinstance(self._classes, State) else self._classes

    @_validate_param(file_path="qtmui.material.formlabel", param_name="color", supported_signatures=Union[State, str])
    def _set_color(self, value):
        """Assign value to color."""
        if value not in self.VALID_COLORS and not isinstance(value, str):
            raise ValueError(f"color must be one of {self.VALID_COLORS} or a custom string, got {value}")
        self._color = value

    def _get_color(self):
        """Get the color value."""
        return self._color.value if isinstance(self._color, State) else self._color

    @_validate_param(file_path="qtmui.material.formlabel", param_name="component", supported_signatures=Union[State, str, type(None)])
    def _set_component(self, value):
        """Assign value to component."""
        self._component = value

    def _get_component(self):
        """Get the component value."""
        return self._component.value if isinstance(self._component, State) else self._component

    @_validate_param(file_path="qtmui.material.formlabel", param_name="disabled", supported_signatures=Union[State, bool])
    def _set_disabled(self, value):
        """Assign value to disabled."""
        self._disabled = value
        self._update_enabled_state()

    def _get_disabled(self):
        """Get the disabled value."""
        return self._disabled.value if isinstance(self._disabled, State) else self._disabled

    @_validate_param(file_path="qtmui.material.formlabel", param_name="error", supported_signatures=Union[State, bool])
    def _set_error(self, value):
        """Assign value to error."""
        self._error = value

    def _get_error(self):
        """Get the error value."""
        return self._error.value if isinstance(self._error, State) else self._error
    
    @_validate_param(file_path="qtmui.material.formlabel", param_name="label", supported_signatures=Union[State, str])
    def _set_label(self, value):
        """Assign value to error."""
        self._label = value

    def _get_label(self):
        """Get the error value."""
        return self._label.value if isinstance(self._label, State) else self._label

    @_validate_param(file_path="qtmui.material.formlabel", param_name="filled", supported_signatures=Union[State, bool])
    def _set_filled(self, value):
        """Assign value to filled."""
        self._filled = value

    def _get_filled(self):
        """Get the filled value."""
        return self._filled.value if isinstance(self._filled, State) else self._filled

    @_validate_param(file_path="qtmui.material.formlabel", param_name="focused", supported_signatures=Union[State, bool])
    def _set_focused(self, value):
        """Assign value to focused."""
        self._focused = value

    def _get_focused(self):
        """Get the focused value."""
        return self._focused.value if isinstance(self._focused, State) else self._focused

    @_validate_param(file_path="qtmui.material.formlabel", param_name="required", supported_signatures=Union[State, bool])
    def _set_required(self, value):
        """Assign value to required."""
        self._required = value

    def _get_required(self):
        """Get the required value."""
        return self._required.value if isinstance(self._required, State) else self._required

    @_validate_param(file_path="qtmui.material.formlabel", param_name="sx", supported_signatures=Union[State, List, Dict, Callable, str, type(None)])
    def _set_sx(self, value):
        """Assign value to sx."""
        self._sx = value

    def _get_sx(self):
        """Get the sx value."""
        return self._sx.value if isinstance(self._sx, State) else self._sx

    def _init_ui(self):
        """Initialize the UI based on props."""
        self.setLayout(QHBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)
        self.layout().setSpacing(0)

        children = self._get_children()
        component = self._get_component() or "label"

        # Clear previous widgets
        self._clear_layout()

        if isinstance(children, str):
            text = children + (" *" if self._get_required() else "")
            typography = Typography(variant="button", text=text)
            typography.setEnabled(not self._get_disabled())
            self.layout().addWidget(typography)
            self._widget_references.append(typography)
        elif isinstance(children, QWidget):
            children.setEnabled(not self._get_disabled())
            self.layout().addWidget(children)
            self._widget_references.append(children)
        elif isinstance(children, list):
            for child in children:
                if child is None:
                    continue
                if not isinstance(child, QWidget):
                    raise TypeError(f"Each element in children must be a QWidget, got {type(child)}")
                child.setEnabled(not self._get_disabled())
                self.layout().addWidget(child)
                self._widget_references.append(child)
        elif children is None:
            pass  # No content

        # Apply styles
        self._set_stylesheet()

        # Connect signals
        self._connect_signals()
        self.useEffect(
            self._set_stylesheet,
            [theme.state]
        )
        self.destroyed.connect(self._on_destroyed)


    def _update_enabled_state(self):
        """Update the enabled state of child widgets."""
        for widget in self._widget_references:
            widget.setEnabled(not self._get_disabled())

    def _set_stylesheet(self, component_styled=None):
        """Apply styles based on theme, classes, and sx."""
        self.theme = useTheme()
        component_styled = component_styled or self.theme.components
        form_label_styles = component_styled.get("FormLabel", {}).get("styles", {})
        root_styles = form_label_styles.get("root", {})
        root_qss = get_qss_style(root_styles)

        # Handle color
        color = self._get_color()
        if color in self.VALID_COLORS:
            color_value = getattr(getattr(self.theme.palette, color, {}), "main")
        else:
            color_value = color
        if self._get_error():
            color_value = self.theme.palette.error.main
        elif self._get_disabled():
            color_value = self.theme.palette.text.disabled
        color_qss = f"color: {color_value};"

        # Handle sx
        sx = self._get_sx()
        sx_qss = ""
        if sx:
            if isinstance(sx, (list, dict)):
                sx_qss = get_qss_style(sx, class_name=f"#{self.objectName()}")
            elif isinstance(sx, Callable):
                sx_result = sx()
                if isinstance(sx_result, (list, dict)):
                    sx_qss = get_qss_style(sx_result, class_name=f"#{self.objectName()}")
                elif isinstance(sx_result, str):
                    sx_qss = sx_result
            elif isinstance(sx, str) and sx != "":
                sx_qss = sx

        # Handle classes
        classes = self._get_classes()
        classes_qss = get_qss_style(classes, class_name=f"#{self.objectName()}") if classes else ""

        # Apply MUI classes
        mui_classes = ["MuiFormLabel-root"]
        if self._get_disabled():
            mui_classes.append("Mui-disabled")
        if self._get_error():
            mui_classes.append("Mui-error")
        if self._get_filled():
            mui_classes.append("MuiFormLabel-filled")
        if self._get_focused():
            mui_classes.append("Mui-focused")
        if self._get_required():
            mui_classes.append("MuiFormLabel-required")

        stylesheet = f"""
            #{self.objectName()} {{
                {root_qss}
                {color_qss}
                {classes_qss}
            }}
            {sx_qss}
        """
        self.setStyleSheet(stylesheet)

    def _connect_signals(self):
        """Connect valueChanged signals of State parameters to their slots."""
        if isinstance(self._children, State):
            self._children.valueChanged.connect(self._on_children_changed)
        if isinstance(self._classes, State):
            self._classes.valueChanged.connect(self._on_classes_changed)
        if isinstance(self._color, State):
            self._color.valueChanged.connect(self._on_color_changed)
        if isinstance(self._component, State):
            self._component.valueChanged.connect(self._on_component_changed)
        if isinstance(self._disabled, State):
            self._disabled.valueChanged.connect(self._on_disabled_changed)
        if isinstance(self._error, State):
            self._error.valueChanged.connect(self._on_error_changed)
        if isinstance(self._filled, State):
            self._filled.valueChanged.connect(self._on_filled_changed)
        if isinstance(self._focused, State):
            self._focused.valueChanged.connect(self._on_focused_changed)
        if isinstance(self._required, State):
            self._required.valueChanged.connect(self._on_required_changed)
        if isinstance(self._sx, State):
            self._sx.valueChanged.connect(self._on_sx_changed)

    def _on_children_changed(self):
        """Handle changes to children."""
        self._set_children(self._children)
        self._init_ui()
        self._set_stylesheet()

    def _on_classes_changed(self):
        """Handle changes to classes."""
        self._set_classes(self._classes)
        self._set_stylesheet()

    def _on_color_changed(self):
        """Handle changes to color."""
        self._set_color(self._color)
        self._set_stylesheet()

    def _on_component_changed(self):
        """Handle changes to component."""
        self._set_component(self._component)
        self._init_ui()
        self._set_stylesheet()

    def _on_disabled_changed(self):
        """Handle changes to disabled."""
        self._set_disabled(self._disabled)
        self._set_stylesheet()

    def _on_error_changed(self):
        """Handle changes to error."""
        self._set_error(self._error)
        self._set_stylesheet()

    def _on_filled_changed(self):
        """Handle changes to filled."""
        self._set_filled(self._filled)
        self._set_stylesheet()

    def _on_focused_changed(self):
        """Handle changes to focused."""
        self._set_focused(self._focused)
        self._set_stylesheet()

    def _on_required_changed(self):
        """Handle changes to required."""
        self._set_required(self._required)
        self._init_ui()
        self._set_stylesheet()

    def _on_sx_changed(self):
        """Handle changes to sx."""
        self._set_sx(self._sx)
        self._set_stylesheet()

    def _clear_layout(self):
        """Remove all widgets from the layout."""
        if self.layout():
            while self.layout().count():
                item = self.layout().takeAt(0)
                if item.widget():
                    item.widget().setParent(None)
            self._widget_references.clear()

    def _on_destroyed(self):
        """Clean up connections when the widget is destroyed."""
        if hasattr(self, "theme"):
            self.theme.state.valueChanged.disconnect(self._set_stylesheet)
        if isinstance(self._children, State):
            self._children.valueChanged.disconnect(self._on_children_changed)
        if isinstance(self._classes, State):
            self._classes.valueChanged.disconnect(self._on_classes_changed)
        if isinstance(self._color, State):
            self._color.valueChanged.disconnect(self._on_color_changed)
        if isinstance(self._component, State):
            self._component.valueChanged.disconnect(self._on_component_changed)
        if isinstance(self._disabled, State):
            self._disabled.valueChanged.disconnect(self._on_disabled_changed)
        if isinstance(self._error, State):
            self._error.valueChanged.disconnect(self._on_error_changed)
        if isinstance(self._filled, State):
            self._filled.valueChanged.disconnect(self._on_filled_changed)
        if isinstance(self._focused, State):
            self._focused.valueChanged.disconnect(self._on_focused_changed)
        if isinstance(self._required, State):
            self._required.valueChanged.disconnect(self._on_required_changed)
        if isinstance(self._sx, State):
            self._sx.valueChanged.disconnect(self._on_sx_changed)